      program drop_2d

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c---------------------------------------------
c Hydrostatic shape of a sessile or pendant
c 2-D drop resting on a horizontal plate,
c for a specified drop area and contact angle.
c
c The numerical procedure is similar to that for the 
c axisymmetric drop described in section 4.4
c of Pozrikidis (1997, pp. 173-177)
c
c The shooting parameter "shp" is the curvature of the
c interface in the xy plane at the top of the drop.
c
c This formulation uses secant iterations.
c
c Legend:
c ------
c
c x,y:  coordinates of interfacial marker points.
c shp:  shooting parameter.
c Isp:  orientation index.
c gac:  magnitude of the acceleration of gravity.
c psi:  interface slope angle.
c error:  residual of the object function involving
c         the drop area.
c tol:  toleration for shooting iterations.
c Itmax: maximum number of shootings
c alpha: contact angle
c
c Gravity is directed toward the negative x-axis
c
c Words of wisdom:
c ----------------
c
c If iterations fail to convergence,
c try reducing the relaxation parameter "relax"
c
c---------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(800),y(800)
      Dimension shp(300),error(300)

      Parameter (tol=0.0000001,Itmax=1000,relax = 1.0)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0

      piq = 0.25D0*pi
      pih = 0.50D0*pi
      pi2 = 2.00D0*pi

      null = 0
      none = 1
      ntwo = 2

      zero = 0.0D0
      one  = 1.0D0
      onem =-1.0D0
      two  = 2.0D0
      twom =-2.0D0

c---
c print wall
c located at x = 0
c---

      open (1,file="PLOTDAT",status="unknown")

      write (1,100) Ntwo
      write (1,100) none,twom,zero
      write (1,100) ntwo,two ,zero

c---
c Preferences
c---

      write (6,*)
      write (6,*) " Choose the data input mode"
      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 to read data from file: drop_2d.dat"
      write (6,*) " 2 to type in data"
      write (6,*) " --------------------------------------"
      read  (5,*) Ienrd

      If(Ienrd.eq.0) Go to 99

c------------------------
      If(Ienrd.eq.2) then
c------------------------

      write (6,*)
      write (6,*) 'Please enter 1 for a sessile drop'
      write (6,*) '            -1 for a pendant drop'
      write (6,*) '                       0  to quit'
      write (6,*) '----------------------------------'
      read  (5,*) Jsp

      If(Jsp.eq.0) Go to 99

      write (6,*)
      write (6,*) 'Enter the acceleration of gravity'
      write (6,*) '---------------------------------'
      read  (5,*) gac

      write (6,*)
      write (6,*) 'Please enter the interfacial tension'
      write (6,*) '------------------------------------'
      read  (5,*) gamma

      write (6,*)
      write (6,*) 'Please enter the density of the drop'
      write (6,*) '------------------------------------'
      read  (5,*) rhod

      write (6,*)
      write (6,*) 'Enter the density of the ambient fluid'
      write (6,*) '--------------------------------------'
      read  (5,*) rhoa

      write (6,*)
      write (6,*) 'Please enter the drop area'
      write (6,*) '--------------------------'
      read  (5,*) area

      write (6,*)
      write (6,*) 'Please enter the contact angle'
      write (6,*) '            in multiples of pi'
      write (6,*) '------------------------------'
      read  (5,*) alpha

      write (6,*)
      write (6,*) 'Enter the number of interfacial marker points'
      write (6,*) '---------------------------------------------'
      read  (5,*) npts

      write (6,*)
      write (6,*) 'Please enter a small value for'
      write (6,*) '        the shooting parameter'
      write (6,*) '------------------------------'
      read  (5,*) epsilon

c----------
       Else
c----------

        open (2,file="drop_2d.dat")

         read (2,*) Jsp
         read (2,*) gac
         read (2,*) gamma
         read (2,*) rhod
         read (2,*) rhoa
         read (2,*) area
         read (2,*) alpha
         read (2,*) npts
         read (2,*) epsilon

        close (2)

c----------
      End If
c----------

c--------
c prepare
c--------

      npts1 = npts+1
      drho  = rhod-rhoa              ! density difference

      capls = gamma/(gac*abs(drho))  ! square of the capillary length

      Isp = 1                        ! Isp is an orientation index
      If(drho.lt.0) Isp = -Isp
      If(Jsp.eq.-1) Isp = -Isp

      alpha = alpha*pi

c-------------------------------------------------
c to start, assume drop shape is a truncated circle
c and compute the circle radius "a" as a function of the
c area and contact angle
c-------------------------------------------------

      a = sqrt(area/(alpha-0.5*sin(2.0*alpha)))

      shp(1) = 1.0/a

      write (6,*)
      write (6,*) " Shooting variable is drop curvature at top"
      write (6,*) " Initial shooting parameterset to: ",shp(1)
c     write (6,*)
c     write (6,*) " Please enter 1 to proceed or 0 to change it"
c     write (6,*) "-------------------------------------------"
c     read  (5,*) Ichange

      Ichange = 1

      If(Ichange.eq.0) then
       write (6,*)
       write (6,*) " Please enter the new value"
       read  (5,*) shp(1)
      End If

c-----------------------
c prepare for displaying
c-----------------------

  94  Continue

      write (6,*) 
      write (6,120) alpha
      write (6,*) 
      write (6,*) " Area, shooting parameter, error"
      write (6,*) 

c---
c Compute initial solution of the odes 
c to start-up the secant method
c---

      dpsi  = alpha/(npts1-1.0)

      Ic = 1     ! counter

      call drop_2d_ode
     +
     +   (npts
     +   ,capls
     +   ,Isp
     +   ,dpsi
     +   ,shp(Ic)
     +   ,x,y
     +   ,area_sh
     +   )

      error(Ic) = area_sh - area
      err       = abs(error(Ic))      ! to err is human

      write (6,100) Ic,area_sh,shp(Ic),err

c---
c second start-up solution
c---

      Ic = 2

      shp(2) = shp(1)+0.01

      call drop_2d_ode
     +
     +    (npts
     +    ,capls,Isp
     +    ,dpsi
     +    ,shp(Ic)
     +    ,x,y
     +    ,area_sh
     +    )

      error(Ic) = area_sh - area
      err       = abs(error(Ic))

      write (6,100) Ic,area_sh,shp(Ic),err

c---
c iterate on "shp" using the secant method until convergence
c---

      Do while(err.gt.tol)

       Ic = Ic+1

       If(Ic.gt.Itmax) then
        write(6,*) 
        write(6,*) 'Sorry: Iterations did not converge'
        write(6,*) 
        Go to 97
       End If

c----------------
c Secant updating
c----------------

       Icb = Ic-2
       Ica = Ic-1

       dedc = (error(Ica)-error(Icb))
     +       /(  shp(Ica)-  shp(Icb))

       shp(Ic) = shp(Ica)-relax*error(Ica)/dedc

      call drop_2d_ode
     +
     +     (npts
     +     ,capls,Isp
     +     ,dpsi
     +     ,shp(Ic)
     +     ,x,y
     +     ,area_sh
     +     )

       error(Ic) = area_sh-area
       err       = abs(error(Ic))

       write (6,100) Ic,area_sh,shp(Ic),err


      Itry = Itry + 1

      End Do                           ! iteration on shp

 97   Continue

c---
c printing session
c
c Draw drop interface
c and its reflection with respect to x = 0
c---

      alpha = alpha/pi     ! unscale for printing

      shift = x(npts1)     ! shift to reset the origin at the wall

      write (1,*) npts1,alpha,area

      Do i = 1,npts1
       xpr = Jsp*(x(i)-shift)
       ypr = y(i)
       write (1,100) i,ypr,xpr
      End Do

      write (1,*) npts1,alpha,area
      Do i = 1,npts1
       xpr = Jsp*(x(i)-shift)
       ypr = -y(i)
       write (1,100) i,ypr,xpr
      End Do

      alpha = alpha*pi      ! scale

c----
c return for another shape
c---

      write (6,*)
      write (6,*)  " Current contact angle is:",alpha
      write (6,*)
      write (6,*)  " Please enter the new contact angle"
      write (6,*)  "              in multiples of pi"
      write (6,*)  "  0 to quit"
      write (6,*)  " -----------"
      read  (5,*) alpha
 
      If(alpha.eq.0) Go to 99
 
      shp(1) = shp(Ic)

      Go to 94

c-----------
c pack it in
c-----------

  99  Continue

      write (1,*) null
      close(1)

c-----
c Done
c-----

 100  Format (1x,i3,9(2x,f15.10))
 120  Format ("Contact angle: ",f15.10)

      Stop
      End
