  close all
  clear all

%======================================================
% FDLIB
%
% Hydrostatic shape of an axisymmetric
% sessile drop resting on a horizontal plate
% or pendant drop hanging underneath a horizontal plate
% for a specified volume and contact angle
%======================================================

  Jsp=-1; % pendant
  Jsp= 1; % sessile

  a      = 1.0;        % equivalent drop radius

  gac    = 1.0;        % acceleration of gravity
  rhod   = 1.0;        % density of the drop
  rhoa   = 0.0;        % density of the ambient fluid

  alpha = 0.75*pi;   % contact angle
  alpha = 0.25*pi;   % contact angle
  alpha = 0.50*pi;   % contact angle

  npts  = 8*32;     %  number of interfacial markers

  epsilon = 0.01; % for the shooting method

  maxiter = 32;
  tol = 0.00000001;

%---
% prepare
%---

  volume = 4.0*pi/3.0*a^3;   % drop volume
  Drho = rhod-rhoa;
  cosa = cos(alpha);

%----------------------------------
% assume the drop is a truncated sphere
% and compute the sphere radius rsph
% in terms of the volume and contact angle
%----------------------------------

  rsph = (3.0D0*volume/pi)/(2.0D0+cosa^3-3.0D0*cosa);
  rsph = rsph^(1.0D0/3.0D0);
  B = 2.0/rsph;

%---
% figure
%---

  figure(1)
  hold on
  xlabel('y/a','fontsize',14)
  ylabel('x/a','fontsize',14)
  set(gca,'fontsize',14)
  axis equal

  if(Jsp==1)
   axis([-2.50 2.5 -0.5 2.0])
  else
   axis([-2.50 2.5 -2.5 0.5])
  end
  box on

%----
% prepare
%----

  if(Jsp==1)
   gamma = 16.0; % surface tension
   nloop = 20;
   nloop = 19;
   fcg = 0.75;
  else
   gamma = 16.0; % surface tension
   nloop = 15;  
   fcg = 0.75;
  end

%====
for iloop=1:nloop
%====

  capls = gamma/(abs(Drho)*gac);
  capl  = sqrt(capls);

%---
% compute the shape
%---

  [x,s,B] = drop_ax (Jsp ...
 ...
   ,gac,gamma,rhod,rhoa,volume ...
   ,alpha,npts,epsilon,maxiter,tol ...
   ,B ...
   );

%---
% base radius
%---

  d = -x(npts+1);   % drop height
  b =  s(npts+1);   % drop base radius
  km0 = 0.5*B;

  force = - Jsp*pi*b^2*Drho*gac*d - gamma*2*pi*b*(b*km0-sin(alpha));

  [-force  4*pi*Drho*a^3/3];

%---
% J integral
%---

  J = 0.0;

  for i=1:npts
    wp = (s(i+1)-s(i))/(x(i+1)-x(i));
    sm = 0.5*(s(i+1)+s(i));
    J = J + 1.0/sm/sqrt(1+wp^2)*abs(x(i+1)-x(i));
  end
  bh = b/capl;
  dh = d/capl;

%  [2*sin(0.5*alpha)^2-B*d+J  0.5*dh^2]

  Japproximate = 1/(6.0*bh) * (8.0-(4.0-dh^2)^1.5);
%  [J Japproximate]
%  pause

%---
% shift and plot
%---

  xplt = x+d;
  splt = s;

  figure(1)
  plot( splt,Jsp*xplt,'-k')
  plot(-splt,Jsp*xplt,'-k')

%---
% spherical drop
%---

  Nsph = 64;
  dtheta = alpha/Nsph;

  xc = rsph*cos(alpha);

  for i=1:Nsph+1
   theta   = pi-alpha+(i-1)*dtheta;
   xsph(i) = xc+rsph*cos(theta);
   ysph(i) =    rsph*sin(theta);
  end

%---
% wall
%---

 xwall(1)=-2.5;
 ywall(1)= 0;
 xwall(2)= 2.5;
 ywall(2)= 0;

 if(Jsp==1)
  patch([xwall xwall(2) xwall(1)],[ywall ywall(2)-0.2 ywall(1)-0.2],'y')
  plot( ysph,-xsph,':k')
  plot(-ysph,-xsph,':k')
 else
  patch([xwall xwall(2) xwall(1)],[ywall ywall(2)+0.2 ywall(1)+0.2],'y')
  plot( ysph,xsph,':k')
  plot(-ysph,xsph,':k')
 end
 plot(xwall,ywall,'k')

 gamma = fcg*gamma;

%===
 end % of iloop 
%===

%---
% done
%---
