      program men_2d

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c--------------------------------------------------
c Hydrostatic shape of a 2D meniscus between
c two vertical parallel plates computed
c by the shooting method
c Pozrikidis (1997, pp. 169-171)
c
c The boundary-value problem is:
c
c df/dx = g        (definition)
c dg/dx = (f+h) (1.0+g**2)**1.5 / l**2
c
c see eq. (4.3.14)
c
c The updates are done using the secant method
c
c Legend
c ------
c
c npts: number of points along the interface
c
c tol: tolerance for stopping the secant iterations
c
c Itmax: maximum number of secant iterations
c
c error:  residual of object function: f'(x=b)-cota
c--------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(800),f(800),g(800)
      Dimension h(100),error(100)

      Parameter (tol=0.000001,Itmax=100)

c----------
c constants
c----------

      pi  = 3.14159 265358D0

      piq = 0.25D0*pi
      pih = 0.50D0*pi

      null = 0
      none = 1
      ntwo = 2

      zero = 0.0D0
      one  = 1.0D0
      onem =-1.0D0
      two  = 2.0D0
      twom =-2.0D0

c--------
c prepare
c--------

      open (1,file="PLOTDAT")

c------------
c preferences
c------------

      write (6,*) " Enter:"
      write (6,*) 
      write (6,*) " 0 to quit"
      write (6,*) " 1 to read data from file: men_2d.dat"
      write (6,*) " 2 to type in data"
      write (6,*) " -------------------------------------"
      read  (5,*) Ienrd

      If(Ienrd.eq.0) Go to 99

c------------------------
      If(Ienrd.eq.2) then
c------------------------

      write (6,*)
      write (6,*) " Enter the acceleration of gravity"
      write (6,*) "                         0 to quit"
      write (6,*) " ---------------------------------"
      read  (5,*) gac

      If(gac.eq.0) Go to 99

      write (6,*) 
      write (6,*) " Enter the plate half-separation b"
      write (6,*) " ---------------------------------"
      read  (5,*) b

      write (6,*) 
      write (6,*) " Enter the density of the pool"
      write (6,*) " -----------------------------"
      read  (5,*) rhop

      write (6,*) 
      write (6,*) " Enter the density of the ambient fluid"
      write (6,*) " --------------------------------------"
      read  (5,*) rhoa

      write (6,*) 
      write (6,*) " Please enter the interfacial tension"
      write (6,*) " ------------------------------------"
      read  (5,*) gamma

      write (6,*)
      write (6,*) " Enter contact angle in multiples of pi/2"
      write (6,*) " ----------------------------------------"
      read  (5,*) alpha

      write (6,*) 
      write (6,*) "Enter the number of integration intervals"
      write (6,*) "-----------------------------------------"
      read  (5,*) npts

c----------
       Else
c----------

        open (2,file="men_2d.dat")

         read (2,*) gac
         read (2,*) b
         read (2,*) rhop
         read (2,*) rhoa
         read (2,*) gamma
         read (2,*) alpha
         read (2,*) npts

        close (2)

c----------
      End If
c----------

c-----------------------------
c check the density difference
c-----------------------------

      drho = rhop-rhoa

      If(drho.lt.0.0) then
       write (6,*)
       write (6,*) " Sorry: density of the pool must be higher "
       write (6,*) "        than that of the ambient fluid"
       write (6,*)
       Go to 99
      End If

c--------
c prepare
c--------

      npts1 = npts+1
      bm = -b

      capls = gamma/(gac*abs(drho))  ! square of capillary length
                                     
      alpha = pih*alpha

c-----------------------------------------
c initial guess for h
c computed by assuming a circular interface
c-----------------------------------------

      h(1) = capls*Dcos(alpha)/b

      write (6,*)
      write (6,*) " Shooting parameter is the meniscus rise"
      write (6,*) " Initial value set to: ",h(1)
c     write (6,*)
c     write (6,*) " Please 0 to proceed, 1 to change it"
c     write (6,*) " -----------------------------------"
c     read  (5,*) Ichange

      Ichange = 0

      If(Ichange.eq.1) then
       write (6,*)
       write (6,*) " Please enter the new value"
       read (5,*) h(1)
      End If

c------------------
c more preparations
c------------------

  94  Continue

      If(abs(alpha-pih).lt.0.00001) then
       cota = 0.0D0
      Else
       cota  = 1.0D0/Dtan(alpha)
      End If

c-------------------
c prepare to display
c-------------------

      write (6,*)
      write (6,*) " Radius, shooting parameter, error"
      write (6,*)
                             
c--------------------------------------------
c compute first start-up solution of the odes
c to start up the secant method
c--------------------------------------------

      Ic = 1                 ! counter

      call men_2d_ode
     +
     +  (npts
     +  ,capls
     +  ,b
     +  ,h(Ic)
     +  ,x,f,g
     +  )

      error(Ic) = g(npts1)-cota
      err       = abs(error(Ic))

      write (6,100) Ic,g(npts1),h(Ic),err
                                           
c-------------------------
c second start-up solution
c-------------------------

      Ic = 2

      h(2) = h(1)+0.01

      call men_2d_ode
     +
     +   (npts
     +   ,capls
     +   ,b
     +   ,h(Ic)
     +   ,x,f,g
     +   )

      error(Ic) = g(npts1)-cota
      err       = abs(error(Ic))

      write (6,100) Ic,g(npts1),h(Ic),err
                                       
c-------------------------------------------------------
c iterate on h using the secant method until convergence
c-------------------------------------------------------

      Iconverge = 0

      Do while(err.gt.tol)

       Ic = Ic+1

       If(Ic.gt.Itmax) then
        write (6,*)
        write (6,*) 'Sorry: iterations did not converge'
        write (6,*)
        Iconverge = 1
        Go to 97
       End If

c---
c Secant update
c---

      Icb = Ic-2
      Ica = Ic-1

      dedh = (error(Ica)-error(Icb))/(h(Ica)-h(Icb))

      h(Ic) = h(Ica)-error(Ica)/dedh

       call men_2d_ode
     +
     +   (npts
     +   ,capls
     +   ,b
     +   ,h(Ic)
     +   ,x,f,g
     +   )

      error(Ic) = g(npts1)-cota
      err       = abs(error(Ic))

      write (6,100) Ic,g(npts1),h(Ic),err

      End Do

  97  Continue

      cr = h(Ic)     ! capillary rise

c-----------------
c printing session
c-----------------

c----
c print vertical plates
c----

      height  = abs(cr)+abs(f(npts1))+0.1
      heightm = -height

      write (1,100) Ntwo
      write (1,100) none, b,heightm
      write (1,100) ntwo, b,height

      write (1,100) Ntwo
      write (1,100) none,bm,heightm
      write (1,100) ntwo,bm,height

c---
c print pool
c---

      bpo  = b+1.0
      bpom = -bpo

      write (1,100) Ntwo
      write (1,100) none,  b,zero
      write (1,100) ntwo,bpo,zero

      write (1,100) Ntwo
      write (1,100) none,bpom,zero
      write (1,100) ntwo,  bm,zero

c----------------------------------
c print meniscus and its reflection
c with respect to the mid-plane
c----------------------------------

       shift = -cr   ! shift to reset the origin at the pool

       write (1,100) npts1

       Do i=1,npts1
        write (1,100) i,x(i),f(i)-shift
       End Do

       write (1,100) npts1

       Do i=1,npts1
        write (1,100) i,-x(i),f(i)-shift
       End Do

c-------
c repeat
c-------

      write (6,*) 
      write (6,*) " Enter a new contact angle"
      write (6,*) " in multiples of pi/2"
      write (6,*) 
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) alpha

      If(alpha.eq.0) Go to 99

      If(Iconverge.eq.0) then
       h(1) = cr
      Else
       h(1) = capls*cos(alpha)/b
      End If

      Go to 94

c-----
c Done
c-----

  99  Continue

      write (1,100) null
      close (1)

 100  Format (1x,i3,5(2x,f15.10))

      Stop
      End
