function [Q,u,yI,uI] = films ...
...
         (NLR,thick ...
         ,visc,den,gac ...
         ,beta ...
         ,y)

%=========================================
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved
%
% This program is to be used only under the
% stipulations of the licensing agreement
%=========================================

%=========================================
% Multi-film flow down an inclined plane
% located at y=0
%
% Film numbered 1 is adjacent to the plane
%
% The velocity in the ith film is given by:
%
% u(i) = af(i) + bf(i)*y - gf(i)*y^2
%
% SYMBOLS:
% -------
%
% NLR:    Number of films
% yI(i):  y-position of the ith interface
% uI(i):  velocity at the ith interface
% Q(i);   flow rate of ith film
% beta:   plane inclination angle
% gac:    magnitude of the acceleration of gravity
%=========================================

%----------------------------
% set the interface positions
%----------------------------

yI(1) = thick(1);

for i=2:NLR
    yI(i) = yI(i-1)+thick(i);
end

%-----------------------------------------
% The velocity in the ith film is given by:
% u(i) = af(i) + bf(i)*y - gf(i)*y^2
% compute gf(i)
%-----------------------------------------

fc = 0.5*gac*sin(beta);

for i=1:NLR
  gf(i) = fc*den(i)/visc(i);
end

%---
% compute bf(i) by recursion
% starting from the top
% and requiring continuity of shear stress
%---

bf(NLR) = 2.0*gf(NLR)*yI(NLR);

for i=NLR-1:-1:1
   bf(i) = 2.0*gf(i)*yI(i)  ...
       + visc(i+1)/visc(i)*(bf(i+1)-2.0*gf(i+1)*yI(i));
end

%---
% compute af(i) by recursion
% starting from the bottom
% and requiring continuity of velocity
%---

af(1) = 0.0;

for i=1:NLR-1
   af(i+1) = af(i)+( (bf(i)-bf(i+1)) ...
          -(gf(i)-gf(i+1))*yI(i)) * yI(i);
end

%---
% compute the interfacial velocities
%---

for i=1:NLR
  uI(i) = af(i) + (bf(i)-gf(i)*yI(i)) * yI(i);
end

%----
% compute the flow rates in the ith film
% is confined between i-1 and i interfaces
%----

  yI0 = 0.0;
  Q(1) = af(1)*(yI(1)-yI0) ...
        +bf(1)*(yI(1)^2-yI0^2)/2.0 ...
        -gf(1)*(yI(1)^3-yI0^3)/3.0;

 for i=2:NLR
   Q(i) = af(i)*(yI(i)-yI(i-1)) ...
         +bf(i)*(yI(i)^2-yI(i-1)^2)/2.0 ...
         -gf(i)*(yI(i)^3-yI(i-1)^3)/3.0;
 end

%--------------------------
% compute the velocity at y
%--------------------------

% identify the host layer:

for i=1:NLR
  if(y<yI(i)) break; end
end

if(i>NLR) i = NLR; end
     
%---
% evaluate the velocity
%---

u = af(i) + (bf(i)-gf(i)*y)*y

%-----
% Done
%-----

return
