      subroutine tube_ann_ml
     +
     +  (NLR
     +  ,a1
     +  ,a2
     +  ,thick
     +  ,visc
     +  ,den
     +  ,gac
     +  ,V1
     +  ,V2
     +  ,chi
     +  ,beta
     +  ,s
     +  ,sI
     +  ,uI
     +  ,xi1,xi2
     +  ,Q
     +  ,u
     +  )

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c==========================================

c==========================================
c Multi-layer core-annular flow through an annular tube
c
c Layer numbered 1 is adjacent to the inner cylinder
c Layer numbered NLR is adjacent to the outer cylinder
c
c The velocity in the ith layer is given by:
c
c u(i) = af(i) + bf(i)*ln(sigma/a1) - gf(i)*sigma**2
c
c SYMBOLS:
c --------
c
c NLR:       Number of layers
c thick(i):  thickness of the ith layer
c sI(i):     sigma position of the ith interface
c uI(i):     velocity of the ith interface
c xi1(i):    shear rate on lower side of the ith interface
c xi2(i):    shear rate on upper side of the ith interface
c Q(i):      flow rate of ith layer
c beta:      tube inclination angle
c gac:       magnitude of the acceleration of gravity
c chi:       negative of the pressure gradient
c==========================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension gf(10),af(10),bf(10)
      Dimension sI(10),uI(10),xi1(10),xi2(10),Q(10)

c---
c constants
c---

      pi  = 3.14159 265358D0
      pi2 = 2.0D0*pi

c---
c interface radial positions
c---

      sI(1) = a1+thick(1)

      Do i=2,NLR
       sI(i) = sI(i-1)+thick(i)
      End Do

c-----------------------------------------
c the velocity in the ith layer is given by:
c
c u(i) = af(i) + bf(i)*ln(sigma/a1) - gf(i)*sigma**2
c
c compute gf(i)
c-----------------------------------------

      fc = gac*sin(beta)

      Do i=1,NLR
        gf(i) = 0.25D0*(chi+fc*den(i))/visc(i)
      End Do

c==========================================
c In the first pass, set bf(1) equal to 0
c and compute the residual of the inner cylinder boundary
c condition: error0
c
c In the second pass, set bf(1) set bf(1) equal to 1
c and compute the residual of the inner cylinder boundary
c condition: error1
c
c In the third pass, set the proper value:
c
c bf(1) = error0/(error0-error1)
c==========================================

      bf(1) = 0.0

      Do Ipass=1,3

c---
c compute bf(i) by recursion
c starting from the inner tube and
c requiring continuity of shear stress
c across the interfaces
c---

      Do i=2,NLR
        bf(i) =    2.0*gf(i) *sI(i-1)**2
     +  + visc(i-1)/visc(i)*(bf(i-1)-2.0*gf(i-1)*sI(i-1)**2) 
      End Do

c---
c compute af(NLR) to satisfy the non-slip condition
c at the outer cylinder
c---

      af(NLR) = V2 - bf(NLR)*log(a2/a1) + gf(NLR)*a2**2

c---
c compute af(i) by recursion
c starting from the outer tube and
c requiring continuity of velocity
c across the interfaces
c---

      Do i=NLR-1,1,-1
       af(i) = af(i+1)+(bf(i+1)-bf(i))*log(sI(i)/a1)
     +               -(gf(i+1)-gf(i))*sI(i)**2
      End Do

c---
c error in the no-slip condition on the inner wall
c error = u(s=a1)-V1
c---

      if(Ipass.eq.1) then

c       error0 = af(1) + bf(1)*log(a1/a1) - gf(1)*a1**2 - V1
        error0 = af(1) - gf(1)*a1**2 - V1
        bf(1)  = 1.0

      elseif(Ipass.eq.2) then

c       error1 = af(1) + bf(1)*log(a1/a1) - gf(1)*a1**2 - V1
        error1 = af(1) - gf(1)*a1**2 - V1
        bf(1)  = error0/(error0-error1)

      end if

c---
      End Do
c---

c-----
c Done with Ipass
c-----

c---
c error2 should be equal to zero
c---

c     error2 = af(1) + bf(1)*log(a1/a1) - gf(1)*a1**2-V1
c     error2 = af(1) - gf(1)*a1**2-V1
c     write (6,*) "error2=",error2

c---
c compute the interfacial velocities
c and shear rates
c---

      Do i=1,NLR-1
       uI(i) = af(i) + bf(i)*log(sI(i)/a1) - gf(i)*sI(i)**2
       xi1(i) = bf(i) /sI(i)-2.0*gf(i) *sI(i)
       xi2(i) = bf(i+1)/sI(i)-2.0*gf(i+1)*sI(i)
      End Do

c----
c compute the flow rates
c the ith layer is confined between i-1 and i interfaces
c----

       sI1s  = sI(1)**2
       sI1as = a1**2
       Q(1) = af(1)*(sI1s-sI1as)/2.0
     +       +bf(1)*0.5*(sI1s *(log(sI(1)/a1)-0.5)+0.5*sI1as)
     +       -gf(1)*(sI1s**2-sI1as**2)/4.0
       Q(1) = pi2*Q(1)

      Do i=2,NLR
       ia = i-1
       sIis  = sI(i )**2
       sIias = sI(ia)**2
       Q(i) = af(i)*(sIis-sIias)/2.0
     +       +bf(i)*0.5*(sIis *(log(sI(i )/a1)-0.5)
     +                  -sIias*(log(sI(ia)/a1)-0.5))
     +       -gf(i)*(sIis**2-sIias**2)/4.0
       Q(i) = pi2*Q(i)
      End Do

c--------------------------
c compute the velocity at y
c--------------------------

c---
c identify the host layer
c numbered j
c---

      i = 1

 92   Continue

      if(i.gt.NLR) then
       i = NLR
       Go to 93
      end if

      if(s.gt.sI(i)) then
       i = i+1
       Go to 92
      end if

      j = i

  93  Continue
     
c---
c evaluate the velocity at s
c---

      u = af(j)+bf(j)*log(s/a1)-gf(j)*s**2

c---
c Done
c---

      Return
      End
