function [sI,uI,xi1,xi2,Q,u] = tube_ann_ml ...
...
        (NLR ...
        ,a1 ...
        ,a2 ...
        ,thick ...
        ,visc ...
        ,den ...
        ,gac ...
        ,V1 ...
        ,V2 ...
        ,chi ...
        ,beta ...
        ,s ...
        )

%==========================================
% Multi-layer core-annular flow through an annular tube
%
% Layer numbered 1 is adjacent to the inner cylinder
% Layer numbered NLR is adjacent to the outer cylinder
%
% The velocity in the ith layer is given by:
%
% u(i) = af(i) + bf(i)*ln(sigma/a1) - gf(i)*sigma^2
%
% SYMBOLS:
% --------
%
% NLR:       Number of layers
% thick(i):  thickness of the ith layer
% sI(i):     sigma position of the ith interface
% uI(i):     velocity of the ith interface
% xi1(i):    shear rate on lower side of the ith interface
% xi2(i):    shear rate on upper side of the ith interface
% Q(i):      flow rate of ith layer
% beta:      tube inclination angle
% gac:       magnitude of the acceleration of gravity
% chi:       negative of the pressure gradient
%==========================================

%---
% interface radial positions
%---

sI(1) = a1+thick(1);

for i=2:NLR
   sI(i) = sI(i-1)+thick(i);
end

%-----------------------------------------
% the velocity in the ith layer is given by:
%
% u(i) = af(i) + bf(i)*ln(sigma/a1) - gf(i)*sigma^2
%
% compute gf(i)
%-----------------------------------------

fc = gac*sin(beta);

for i=1:NLR
 gf(i) = 0.25D0*(chi+fc*den(i))/visc(i);
end

%==========================================
% In the first pass, set bf(1) equal to 0
% and compute the residual of the inner cylinder boundary
% condition: error0
%
% In the second pass, set bf(1) equal to 1
% and compute the residual of the inner cylinder boundary
% condition: error1
%
% In the third pass, set the proper value:
%
% bf(1) = error0/(error0-error1)
%==========================================

bf(1) = 0.0;

for Ipass=1:3

%---
% compute bf(i) by recursion
% starting from the inner tube and
% requiring continuity of shear stress
% across the interfaces
%---

for i=2:NLR
  bf(i) =   2.0*gf(i) *sI(i-1)^2 ...
   + visc(i-1)/visc(i)*(bf(i-1)-2.0*gf(i-1)*sI(i-1)^2); 
end

%---
% compute af(NLR) to satisfy the non-slip condition
% at the outer cylinder
%---

af(NLR) = V2 - bf(NLR)*log(a2/a1) + gf(NLR)*a2^2;

%---
% compute af(i) by recursion
% starting from the outer tube and
% requiring continuity of velocity
% across the interfaces
%---

for i=NLR-1:-1:1
  af(i) = af(i+1)+(bf(i+1)-bf(i))*log(sI(i)/a1) ...
                -(gf(i+1)-gf(i))*sI(i)^2;
end

%---
% error in the no-slip condition on the inner wall
% error = u(s=a1)-V1
%---

      if(Ipass==1) 

        error0 = af(1) - gf(1)*a1^2 - V1;
        bf(1)  = 1.0;

      elseif(Ipass==2) 

        error1 = af(1) - gf(1)*a1^2 - V1;
        bf(1)  = error0/(error0-error1);

      end

%---
end % of Ipass
%---

%---
% compute the interfacial velocities
% and shear rates
%---

for i=1:NLR-1
  uI(i) = af(i) + bf(i)*log(sI(i)/a1) - gf(i)*sI(i)^2;
  xi1(i) = bf(i)/sI(i)-2.0*gf(i) *sI(i);
  xi2(i) = bf(i+1)/sI(i)-2.0*gf(i+1)*sI(i);
end

%----
% compute the flow rates
% the ith layer is confined between i-1 and i interfaces
%----

sI1s  = sI(1)^2;
sI1as = a1^2;
Q(1) = af(1)*(sI1s-sI1as)/2.0 ...
           +bf(1)*0.5*(sI1s *(log(sI(1)/a1)-0.5)+0.5*sI1as) ...
                  -gf(1)*(sI1s^2-sI1as^2)/4.0;
Q(1) = 2.0*pi*Q(1);

for i=2:NLR
       ia = i-1;
       sIis  = sI(i )^2;
       sIias = sI(ia)^2;
       Q(i) = af(i)*(sIis-sIias)/2.0 ...
             +bf(i)*0.5*( sIis *(log(sI(i )/a1)-0.5)...
                         -sIias*(log(sI(ia)/a1)-0.5))...
             -gf(i)*(sIis^2-sIias^2)/4.0;
       Q(i) = 2.0*pi*Q(i);
end

%--------------------------
% compute the velocity at s
%--------------------------

%---
% identify the host layer
%---

 for i=1:NLR
   if(s<sI(i))
     break
   end
 end

 if(i>NLR) i=NLR; end

%---
% evaluate the velocity at s
%---

u = af(i) + bf(i)*log(s/a1) - gf(i)*s^2;

%---
% Done
%---

return
