      program tube_ann_ml_dr

c==================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c==================================

c==================================
c Multi-layer core-annular flow through an annular tube
c driven by gravity or a pressure gradient.
c
c Film numbered 1 is adjacent to the inner wall.
c Film numbered NLR is adjacent to the outer wall.
c
c The velocity in the ith film is given by:
c
c u(i) = af(i)+bf(i)*ln(sigma/a1)-gf(i)*sigma**2
c
c SYMBOLS:
c --------
c
c NLR:       Number of layers
c thick(i):  thickness of the ith layer
c sI(i):     position of the ith interface
c uI(i):  velocity of the ith interface
c xi1(i): shear rate on lower side of the ith interface
c xi2(i): shear rate on upper side of the ith interface
c Q(i):   flow rate of ith layer
c beta:   tube inclination angle
c gac:    magnitude of the acceleration of gravity
c chi:    pressure gradient
c nprof:  number of points per layer for a profile
c==================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension sI(10),ui(10)
      Dimension xi1(10),xi2(10),Q(10)

c----------
c constants
c----------

      pi = 3.14159 265358
      pi2 = 2.0*pi

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3
      Nfour  = 4
      Nfive  = 5

      zero = 0.0

c-------
c  input
c-------

      open (1,file="tube_ann_ml.dat",status="unknown")

        read (1,*) nprof     ! number of points per layer for a profile
        read (1,*) beta
        read (1,*) gac
        read (1,*) a1
        read (1,*) a2
        read (1,*) V1
        read (1,*) V2
        read (1,*) chi
        read (1,*)
        read (1,*) NLR
        read (1,*) (thick(i),i=1,NLR-1)
        read (1,*) (visc (i),i=1,NLR)
        read (1,*) (den  (i),i=1,NLR)

      close (1)

c---
c check for consistency
c and reset the outermost layer thickness
c---

      sum = 0.0
      Do i=1,NLR-1
        sum = sum+thick(i)
      End Do

      thick(NLR) = a2-a1-sum

      if(thick(NLR).lt.0) then
       write (6,*) " tube_ann_ml: layer thicknesses improperly defined"
       stop
      end if

c---
c interface positions
c---

      sI(1) = a1+thick(1)

      Do i=2,NLR
       sI(i) = sI(i-1)+thick(i)
      End Do
c---
c prepare
c---

      beta = pi*beta

      nproftot = NLR*nprof+1   ! total number of points for profile

      open (2,file="tube_ann_ml.out",status="unknown")

      write (2,900) NLR

c---
c paint the inner cylinder
c---

       write (2,900) Nfive
       xprint = -1.2*a2
       yprint =  a1
       write (2,900) None,xprint,yprint
       xprint = 1.2*a2
       yprint = a1
       write (2,900) Ntwo,xprint,yprint
       xprint = 1.2*a2
       yprint = -a1
       write (2,900) Nthree,xprint,yprint
       xprint = -1.2*a2
       yprint = -a1
       write (2,900) Nfour,xprint,yprint
       xprint = -1.2*a2
       yprint =  a1
       write (2,900) Nfive,xprint,yprint

c---
c paint the outer cylinder
c---

       write (2,900) Nfive
       xprint = -1.2*a2
       yprint = a2
       write (2,900) None,xprint,yprint
       xprint =  1.2*a2
       yprint =  a2
       write (2,900) Ntwo,xprint,yprint
       xprint =  1.2*a2
       yprint =  1.2*a2
       write (2,900) Nthree,xprint,yprint
       xprint = -1.2*a2
       yprint =  1.2*a2
       write (2,900) Nfour,xprint,yprint
       xprint = -1.2*a2
       yprint = a2
       write (2,900) Nfive,xprint,yprint

c  reflection

       write (2,900) Nfive
       xprint = -1.2*a2
       yprint = -a2
       write (2,900) None,xprint,yprint
       xprint =  1.2*a2
       yprint = -a2
       write (2,900) Ntwo,xprint,yprint
       xprint =  1.2*a2
       yprint = -1.2*a2
       write (2,900) Nthree,xprint,yprint
       xprint = -1.2*a2
       yprint = -1.2*a2
       write (2,900) Nfour,xprint,yprint
       xprint = -1.2*a2
       yprint = -a2
       write (2,900) Nfive,xprint,yprint

c---
c draw horizontal lines at the interfaces
c---

c--- inner cylinder

       write (2,900) Ntwo
       write (2,900) None,-a2,a1
       write (2,900) Ntwo, a2,a1

c--- outer cylinder

       write (2,900) Ntwo
       write (2,900) None,-a2,a2
       write (2,900) Ntwo, a2,a2

c--- interfaces

      Do i=1,NLR-1

       write (2,900) Ntwo
       write (2,900) None,-a2,sI(i)
       write (2,900) Ntwo, a2,sI(i)

      End Do

c---
c draw a vertical line
c---

      write (2,900) Ntwo
      write (2,900) None,zero,a1
      write (2,900) Ntwo,zero,a2

c---
c profile
c---

c     write (6,*)
c     write (6,*)  " velocity profile"
c     write (6,*)

      write (2,900) nproftot

      Ic = 1           ! counter
      s  = a1

      Do i=1,NLR

       step = thick(i)/(nprof+1.0D0-1.0D0)

       max = nprof
       if(i.eq.NLR) max = nprof+1

       Do j=1,max

         call tube_ann_ml
     +
     +     (NLR
     +     ,a1
     +     ,a2
     +     ,thick
     +     ,visc
     +     ,den
     +     ,gac
     +     ,V1
     +     ,V2
     +     ,chi
     +     ,beta
     +     ,s
     +     ,sI
     +     ,uI
     +     ,xi1,xi2
     +     ,Q
     +     ,u
     +     )

         write (2,900) Ic,u,s
         write (6,900) Ic,u,s

         s = s + step

         Ic = Ic+1

       End Do
      End Do

c---
c end of drawing
c---

      write (2,*) null
      close (2)

c---
c printing session
c---

      write (6,*)
      write (6,*) " Interface radial position, velocity, shear rates"
      write (6,*)

      Do i=1,NLR-1
       write (6,900) i,sI(i),uI(i),xi1(i),xi2(i)
c      ratio = xi1(i)*visc(i)/(xi2(i)*visc(i+1))
c      write (6,*) ratio   ! should be unity
      End Do

      write (6,*)
      write (6,*) " Layer flow rates and mean velocity"
      write (6,*)

      Do i=1,NLR
       umean = Q(i)/(pi*(sI(i)**2-sI(i-1)**2))
       write (6,900) i,Q(i),umean
      End Do

c---
c Done
c---

  99  Continue

 900  Format (1x,i3,9(1x,f10.5))

      Stop
      End
