      program tube_ann_sw_ml_dr

c==========================================
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Multi-layer swirling flow inside an annular tube
c
c Film numbered 1 is adjacent to the inner cylinder
c Film numbered NLR is adjacent to the outer cylinder
c
c The velocity in the ith layer is given by:
c
c u(i) = bf(i)/sigma + cf(i)*sigma
c
c SYMBOLS:
c ------
c
c NLR:       Number of films
c thick(i):  thickness of the ith layer
c a1:        radius of the inner cylinder
c a2:        radius of the outer cylinder
c Omega1:    angular velocity of the inner cylinder
c Omega2:    angular velocity of the outer cylinder
c sI(i):     radial position of the ith interface
c uI(i):     velocity of the ith interface
c xi1(i):    shear rate on lower side of the ith interface
c xi2(i):     shear rate on upper side of the ith interface
c Q(i):       flow rate of ith layer
c==========================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10)
      Dimension sI(10),uI(10)
      Dimension xi1(10),xi2(10),Q(10)

      Parameter (ncyl=64)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

      pi2 = 2.0D0*pi

      Null = 0
      None = 1
      Ntwo = 2
      zero = 0.0

c------------
c preferences
c------------

      open (1,file="tube_ann_sw_ml.dat",status="unknown")

        read (1,*) nprof   ! number of points per layer for a profile
        read (1,*) a1
        read (1,*) a2
        read (1,*) Omega1
        read (1,*) Omega2
        read (1,*)
        read (1,*) NLR
        read (1,*) (thick(i),i=1,NLR-1)
        read (1,*) ( visc(i),i=1,NLR)

      close (1)

c---
c check for consistency
c and set outermost layer thickness
c---

      sum = 0.0D0

      Do i=1,NLR-1
        sum = sum+thick(i)
      End Do

      thick(NLR) = a2-a1-sum

      if(thick(NLR).lt.0) then
       write (6,*) "tube_ann_sw_ml_dr: layer thicknesses are improper"
       stop
      end if

c---
c set the radial interface positions
c---

      sI(1) = a1+thick(1)

      Do i=2,NLR
       sI(i) = sI(i-1)+thick(i)
      End Do

c---
c prepare
c---

      open (2,file="tube_ann_sw_ml.out",status="unknown")
      write (2,900) NLR

c---
c draw the cylinders
c---

      Dth = pi2/(ncyl+1.0D0-1.0D0)

      write (2,900) ncyl+1

      Do i=1,ncyl+1
       thet = (i-1.0)*Dth
       write (2,900) i,a1*cos(thet),a1*sin(thet)
      End Do

      write (2,900) ncyl+1

      Do i=1,ncyl+1
       thet = (i-1.0)*Dth
       write (2,900) i,a2*cos(thet),a2*sin(thet)
      End Do

c---
c draw circles at the interfaces
c---

      Do j=1,NLR-1

      write (2,900) ncyl+1

      Do i=1,ncyl+1
       thet = (i-1.0)*Dth
       write (2,900) i,sI(j)*cos(thet),sI(j)*sin(thet)
      End Do

      End Do

c---
c Draw a vertical line
c---

      write (2,900) Ntwo
      write (2,900) None,zero,a1
      write (2,900) Ntwo,zero,a2

c---
c draw the profile
c---

      write (6,*)
      write (6,*)  " velocity profile:"
      write (6,*)


      nproftot = NLR*nprof+1    ! total number of points for profile

      write (2,900) nproftot+2
      write (2,900) Null,zero,a1

      Ic = 1           ! counter
      s  = a1

c---
      Do i=1,NLR
c---

       step = thick(i)/(nprof+1.0D0-1.0D0)

       max = nprof
       if(i.eq.NLR) max = nprof+1

       Do j=1,max

         call tube_ann_sw_ml
     +
     +    (NLR
     +    ,a1
     +    ,a2
     +    ,thick
     +    ,visc
     +    ,Omega1
     +    ,Omega2
     +    ,s
     +    ,sI
     +    ,uI
     +    ,xi1,xi2
     +    ,Q
     +    ,u
     +    )

         write (2,900) Ic,-u,s
         write (6,900) Ic,-u,s

         s = s+step

         Ic = Ic+1

       End Do
      End Do

      write (2,900) Null,zero,a2

c---
c display session
c---

      write (6,*)
      write (6,*) " Interface radial position, velocity, shear rates"
      write (6,*)

      Do i=1,NLR-1
       write (6,900) i,sI(i),uI(i),xi1(i),xi2(i)
c      ratio = xi1(i)*visc(i)/(xi2(i)*visc(i+1))
c      write (6,*) ratio   ! should be unity
      End Do

      write (6,*)
      write (6,*) " Layer flow rates and mean velocity"
      write (6,*)

      Do i=1,NLR
       umean = Q(i)/(sI(i)-sI(i-1))
       write (6,900) i,Q(i),umean
      End Do

c-----
c Done
c-----

      write (2,*) null
      close (2)

 900  Format (1x,i3,9(1x,f10.5))

      Stop
      End
