      program chan2l_imp

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c===========================================
c Flow of two layers in an inclined channel 
c confined between two parallel plane walls,
c located at y = +-a
c
c Shear-, pressure-, and gravity-driven flow.
c
c Implicit finite-difference method
c with uniform grid spacing.
c
c Legend:
c -------
c
c beta:  inclination angle
c a:     walls are located at y = +-a
c RL:    period
c
c Yunp:  unperturbed position of the interface
c aI0:   initial amplitude of the interface
c hr:     h2/h1
c
c vsr:  lamda (viscosity ratio)
c
c pdrop: pressure drop over one period (positive)
c
c method: 1 for central differences
c         2 for backward differences
c
c obj:  objective function for the Newton iterations
c RJ:  Jacobian matrix
c RB:  RHS of Newton linear system
c
c eps:  Numerical parameter for computation of RJ
c tol:  Tolerance of Newton iterations
c===========================================

      Implicit Double Precision (a-h,o-z)

      Dimension x   (-10:500),y  (-10:500)
      Dimension ynew(-10:500),obj(-10:500)

      Dimension  c(-10:500),DcDt(-10:500)
      Dimension srtn(-10:500)
      Dimension   uI(-10:500)

      Dimension RJ(500,500),RB(500),corr(500)

      Dimension time(0:10000)

      Parameter (pltfc=1.2)   ! for plotting window

c--------------
c common blocks
c--------------

      common/XXYY/x,y
      common/var1/beta,cth0,sth0,a,as,RL
      common/var2/vs1,vs2,vsr,rho1,rho2,Drho,dnr
      common/var3/V1,V2,pdrop,gx,gy
      common/var4/Dx,Dx2,Dxs,Dx23
      common/var5/NSG,NSG1,NSG2,NSG3,NSG4
      common/var6/srtn,uI

c----------
c constants
c----------

      pi  = 3.14159 265358 979323 84 D0
      pih = 0.5D0*pi
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi

      None = 1
      Ntwo = 2
      zero = 0.0D0

c-------------------
c problem parameters
c-------------------

      open (3,file="chan2l_imp.dat")

       read (3,*) a
       read (3,*) beta
       read (3,*) RL
       read (3,*) yIunp
       read (3,*) aI0
       read (3,*)
       read (3,*) asrf0
       read (3,*) phisrf
       read (3,*)
       read (3,*) rho1,rho2
       read (3,*) vs1,vs2
       read (3,*)
       read (3,*) tinit
       read (3,*) cinit
       read (3,*)
       read (3,*) beta_srf
       read (3,*) Dsrf
       read (3,*)
       read (3,*) V1
       read (3,*) V2
       read (3,*) pdrop
       read (3,*) gac
       read (3,*)
       read (3,*) NSG
       read (3,*) Dt
       read (3,*) Nprint
       read (3,*) Nstep
       read (3,*)
       read (3,*) Iread 
       read (3,*)
       read (3,*) method
       read (3,*)
       read (3,*) eps
       read (3,*) tol
       read (3,*)
       read (3,*) IseeJ

      close (3)

c--------
c prepare
c--------

      beta = beta*pi

      cth0 = cos(beta)
      sth0 = sin(beta)
      gx =   gac * sth0
      gy = - gac * cth0

      am = -a
      as = a**2

      vsr = vs2/vs1   ! viscosity ratio

      Drho  = rho1-rho2   ! density difference
      dnr = rho2/rho1   ! density ratio

      betac_srf = 1.0D0-beta_srf     ! surfactant
      alpha_srf = tinit*RL/(vs1*Dsrf) ! surfactant

      shiftx = - RL*cth0   ! graphics
      shifty =   RL*sth0   ! graphics

      open (2,file="chan2l_imp.xy")
      open (3,file="chan2l_imp.out")

c----------------------------
c print inclined channel walls
c----------------------------

      tmp1 =  RLm*cth0 + am*sth0
      tmp2 = -RLm*sth0 + am*cth0
      tmp3 =             am*sth0
      tmp4 =             am*cth0
      tmp5 =  RLm*cth0 + a *sth0
      tmp6 = -RLm*sth0 + a *cth0
      tmp7 =             a *sth0
      tmp8 =             a *cth0

      write (2,103) Ntwo
      write (2,102) None,tmp1,tmp2
      write (2,102) Ntwo,tmp3,tmp4

      write (2,103) Ntwo
      write (2,102) None,tmp5,tmp6
      write (2,102) Ntwo,tmp7,tmp8

c----------------------
c Initial profile
c
c  Iread = 1 : Generate
c          2 : read
c----------------------

c-----------------------
      if(Iread.eq.1) then  ! generate
c-----------------------

        time(0) = 0.0D0

        NSGd = NSG-4
        NSGc = NSG-3
        NSGb = NSG-2
        NSGa = NSG-1

        NSG1 = NSG+1
        NSG2 = NSG+2
        NSG3 = NSG+3
        NSG4 = NSG+4

        wn = pi2/RL           ! wave number
        Dx = RL/(NSG1-1.0D0)

        Do i=-3,NSG4
          x(i) = (i-1.0D0)*Dx
          arg  = wn*x(i)
          y(i) = yIunp + aI0*cos(arg)
          arg  = wn*x(i)- phisrf * pih
          c(i) = cinit + asrf0*Dcos(arg)
        End Do

c---------
      Else    ! read
c---------

        open (4,file='chan2l_imp.inp')

        read (4,*) NSG1,time(0)

        Do j=1,NSG1

          read (4,*) idle,x(j),y(j),c(j)

          x(j) = x(j) - shiftx           ! shift
          y(j) = y(j) - shifty
          tmx  = x(j)*cth0 - y(j)*sth0   ! rotate
          tmy  = x(j)*sth0 + y(j)*cth0
          x(j) = tmx
          y(j) = tmy

          write (6,104) J,x(j),y(j),c(j)

        End Do

        close (4)
c---
c prepare
c---

        NSG  = NSG1-1
        NSG2 = NSG +2
        NSG3 = NSG +3
        NSG4 = NSG +4
c---
c wrap
c---
        x(-3) = x(NSGc)-RL
        y(-3) = y(NSGc)
        x(-2) = x(NSGb)-RL
        y(-2) = y(NSGb)
        x(-1) = x(NSGa)-RL
        y(-1) = y(NSGa)
        x( 0) = x(NSG) -RL
        y( 0) = y(NSG)

        x(NSG2) = x(2)+RL
        y(NSG2) = y(2)
        x(NSG3) = x(3)+RL
        y(NSG3) = y(3)
        x(NSG4) = x(4)+RL
        y(NSG4) = y(4)

c-----------
      End If    ! end of defining the initial profile
c-----------

c----------------------------------
c initial guess for Newton's method
c----------------------------------

      Do i=-3,NSG4
       ynew(i) = y(i)
      End Do

c-----------------------------------------
c prepare for the finite-difference method
c and initialize
c-----------------------------------------

      Dx   = RL/(NSG1-1.0D0)
      Dx2  = 2.0D0*Dx
      Dx23 = 2.0D0*Dx**3
      Dxs  = Dx**2

      Kstep  = 1      ! step counter

      Iprint = Nprint

c-.-.-.-.-.-.-.-.-.-.-.-.-.-
c THE SIMULATION BEGINS HERE 
c-.-.-.-.-.-.-.-.-.-.-.-.-.-


      write (6,*)
      write (6,*) " time, xmax,ymax, xmin, ymin"

  90  Continue             ! will return here after a step

c--------------------------------------
c compute max & min interface elevation
c--------------------------------------

      call elv_max_min
     +
     +  (NSG
     +  ,xmax,ymax
     +  ,xmin,ymin
     +  )

      amp = 0.5D0*(ymax-ymin)/aI0
      amp = Dlog(amp)

      write (6,105) Kstep,time(Kstep),amp,xmax,ymax,xmin,ymin
      write (3,105) Kstep,time(Kstep),amp,xmax,ymax,xmin,ymin

      if(Kstep.eq.Nstep+1) Go to 99

c----------------
c surface tension
c from surfactant concentration
c----------------

      Do i=-3,NSG4
       srtn(i) = tinit/betac_srf * (1.0D0-beta_srf*c(i)/cinit)
      End Do

c----------------------------
c printing session:
c
c print interface
c so that it appears inclined
c----------------------------

      if(Iprint.eq.Nprint) then

        write (2,104) NSG1,time(Kstep)

        Do j=1,NSG1
          xpr =  x(j)*cth0 + y(j)*sth0   ! rotate
          ypr = -x(j)*sth0 + y(j)*cth0
          xpr = xpr + shiftx             ! shift
          ypr = ypr + shifty
          write (2,102) J,xpr,ypr,c(j),srtn(j)
          write (6,102) J,xpr,ypr,c(j),srtn(j)
        End Do

        Iprint = 0

      End If

c------------------
c Newton iterations
c------------------

      Iter = 0    ! counter of Newton iterations

  97  Continue

      Iter = Iter+1

c--------------------------------------------------------
c Generate a linear system by numerical differentiation
c
c Compute the left-hand side of the equation:
c dh1/dt+dQ1/dx = 0
c
c Newton system is: RJ * correction = RB
c--------------------------------------------------------

      call chan2l_imp_of
     +
     +   (method
     +   ,Dt
     +   ,x,y
     +   ,ynew
     +   ,obj
     +   )

      Do i=1,NSG
       RB(i) = - obj(i)
      End Do

c---
c compute the Jacobian
c by numerical differentiation
c---

      Do i=1,NSG

        ynew(i) = ynew(i)+eps

        ynew( 0) = ynew(NSG)
        ynew(-1) = ynew(NSGa)
        ynew(-2) = ynew(NSGb)
        ynew(-3) = ynew(NSGc)

        ynew(NSG1) = ynew(1)
        ynew(NSG2) = ynew(2)
        ynew(NSG3) = ynew(3)
        ynew(NSG4) = ynew(4)

        call chan2l_imp_of
     +
     +   (method
     +   ,Dt
     +   ,x,y
     +   ,ynew
     +   ,obj
     +   )

        Do j=1,NSG
          RJ(j,i) = (obj(j)+RB(j))/eps
        End Do
c---
c reset
c---
        ynew(i) = ynew(i)-eps

        ynew( 0) = ynew(NSG)
        ynew(-1) = ynew(NSGa)
        ynew(-2) = ynew(NSGb)
        ynew(-3) = ynew(NSGc)

        ynew(NSG1) = ynew(1)
        ynew(NSG2) = ynew(2)
        ynew(NSG3) = ynew(3)
        ynew(NSG4) = ynew(4)

      End Do

c---------------------
c display the Jacobian
c---------------------

      If(IseeJ.eq.1) then

       write (6,*)
       write (6,*) " chan2l_imp: Newton system:"
       write (6,*)

       Do i=1,NSG
        write (6,105) (RJ(i,j),j=1,NSG),RB(i)
       End Do

      End If

c-----------------
c Newton iteration
c by Gauss elimination
c-----------------

      Isym_g = 0  ! system is not symmetric
      Iwlpvt = 1  ! pivoting enabled

      call gel
     +
     +   (NSG
     +   ,RJ,RB,corr
     +   ,Isym_g
     +   ,Iwlpvt
c    +   ,l,u
     +   ,det
     +   ,Istop
     +   )

      If(Istop.eq.1) then
        write (6,*) " chan2l_imp: gel failed"
        Go to 99
      End If

      cmax = 0.0D0             ! max correction

      Do i=1,NSG
       ynew(i) = ynew(i)+corr(i)
       If(abs(corr(i)).ge.cmax) cmax = abs(corr(i))
c      write (6,105) corr(i)
      End Do

c     write (6,204) Iter,cmax

c-----
c wrap
c-----

      ynew(0)  = ynew(NSG)
      ynew(-1) = ynew(NSGa)
      ynew(-2) = ynew(NSGb)
      ynew(-3) = ynew(NSGc)

      ynew(NSG1) = ynew(1)
      ynew(NSG2) = ynew(2)
      ynew(NSG3) = ynew(3)
      ynew(NSG4) = ynew(4)

      If(cmax.gt.tol) Go to 97     ! for one more iteration

c-------------------------
c End of Newton iterations
c-------------------------

      Do i=-3,NSG4
       y(i) = ynew(i)
      End Do

c----------------------------
c evolution of the surfactant
c----------------------------

      Do i=1,NSG
       i1 = i+1
       ia = i-1
       DcDt(i) = - (uI(i1)*c(i1)-uI(ia)*c(ia))/Dx2
     +           + Ds*(c(i1)-2.0D0*c(i)+c(ia))/Dxs
c      write (6,*) DcDt(i)
      End Do

c----------------------------------
c explicit Euler for the surfactant
c----------------------------------

      Do i=1,NSG
       c(i) = c(i) + Dt*DcDt(i)
      End Do

c-----
c wrap
c-----

      y( 0) = y(NSG)
      y(-1) = y(NSGa)
      y(-2) = y(NSGb)
      y(-3) = y(NSGc)

      y(NSG1) = y(1)
      y(NSG2) = y(2)
      y(NSG3) = y(3)
      y(NSG4) = y(4)

      c( 0) = c(NSG)
      c(-1) = c(NSGa)
      c(-2) = c(NSGb)
      c(-3) = c(NSGc)

      c(NSG1) = c(1)
      c(NSG2) = c(2)
      c(NSG3) = c(3) 
      c(NSG4) = c(4)

c------------------------
c reset counters and time
c------------------------

      Kstep  = Kstep+1
      Iprint = Iprint+1

      time(Kstep) = time(Kstep-1)+Dt

      Go to 90

c-.-.-.-.-.-.-.-.-.-.-.-.-
c THE SIMULATION HAS ENDED
c-.-.-.-.-.-.-.-.-.-.-.-.-

  99  Continue

c-------------------
c Print restart file
c-------------------

      open (7,file="chan2l_imp.restart")      ! restart file

      write (2,104) NSG1,time(Kstep)
      write (7,104) NSG1,time(Kstep)

      Do i=1,NSG1
        xpr =  x(i)*cth0 + y(i)*sth0   ! rotate
        ypr = -x(i)*sth0 + y(i)*cth0
        xpr = xpr + shiftx             ! and shift
        ypr = ypr + shifty
        write (2,102) i,xpr,ypr
        write (7,102) i,xpr,ypr
c       write (6,102) i,xpr,ypr
      End Do

      write (2,104) null
      write (7,104) null

      close (7)

c-------
c finish
c-------

      beta = beta/pi    !  unscale

      write (2,*)
      write (2,*) " chan2l_imp"
      write (2,*)
      write (2,600) a
      write (2,601) beta
      write (2,602) RL
      write (2,603) yIunp
      write (2,604) aI0
      write (2,605) rho1,rho2
      write (2,606) vs1,vs2
      write (2,607) srtn
      write (2,608) V1,V2
      write (2,609) pdrop
      write (2,610) gac
      write (2,611) NSG
      write (2,612) Dt
      write (2,613) Nprint
      write (2,614) method
      write (2,616) eps
      write (2,617) tol

      close (2)

c-----
c Done
c-----

 102  Format (1X,I3,20(1X,F12.8))
 103  Format (1X,I3,1X,F10.5,1X,F10.7)
 104  Format (1X,I3,1X,F12.8,1X,F12.8,1X,I1)
 105  Format (1X,I6,19(1X,F10.7))
 109  Format (" Step: ",I3," out of ",I3,"  TIME = ",f10.5)

 204  Format (1X," Iteration:",I3,1X," Correction:",F12.8)

 501  Format (4(1x,f10.5))
 502  Format (3(1x,f10.5),1x,I1)

 600  Format (' a      = ',f10.5)
 601  Format (' beta   = ',f10.5)
 602  Format (' L      = ',f10.5)
 603  Format (' yIunp  = ',f10.5)
 604  Format (' aI0    = ',f10.5)
 605  Format (' rho1,2 = ',f10.5,1x,f10.5)
 606  Format (' vs 1,2 = ',f10.5,1x,f10.5)
 607  Format (' srtn   = ',f10.5)
 608  Format (' V1,2   = ',f10.5,1x,f10.5)
 609  Format (' pdrop  = ',f10.5)
 610  Format (' gac    = ',f10.5)
 611  Format (' NSG    = ',I3)
 612  Format (' Dt     = ',f10.5)
 613  Format (' Nprint = ',I3)
 614  Format (' Method = ',I3)
 616  Format (' eps    = ',F12.10)
 617  Format (' tol    = ',F12.10)

      Stop
      End
