      program ann2lel0

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c----------------------------------------

c--------------------------------------------------------
c  Instability of two stationary annular layers 
c  confined between two coaxial cylinders.
c
c  The interface has a pre-stressed uniform tension gamma
c  and develops elastic tensions due to the deformation
c
c  Inertial effects are neglected, and the motion occurs
c  under conditions of Stokes flow.
c
c  NUMERICAL METHOD:
c  ----------------
c
c  The determinant of the secular matrix is a quadratic function 
c  of the growth rate "gr", denoted as:
c
c  Det = a*(gr-gref)**2 + b*(gr-gref) + c
c
c  where gref is an arbitrary reference value
c
c  NOTATION:
c  --------
c
c  k:  wave number of the disturbance
c
c  a:  radius of the interface
c  ai: radius of inner cylinder
c  ae: radius of outer cylinder
c
c  visc1: viscosity of the internal fluid
c  visc2: viscosity of the external fluid
c
c  gm0:  unperturbed-state surface tension
c  els:  interface elasticity
c
c  gr:	growth rate: -ikc
c
c  secm(i, j):   secular matrix
c  n:            size of the secular matrix
c
c Icase: Index for classifying configuration as follows:
c
c        1: inner cylinder present, outer cylinder present
c        2: inner cylinder absent, outer cylinder present
c        3: inner cylinder present, outer cylinder absent
c        4: inner cylinder absent, outer cylinder absent
c
c  To make the inner cylinder disappear, set ai = 0.0
c  To make the outer cylinder disappear, set ae = 0.0
c-----------------------------------------------

      Implicit Double Precision(a-h,k,o-z)

      Dimension secm(8,8)
      Dimension rhs(8),sol(8)    ! for Gauss elimination only

      Parameter (small=0.000001)
      Parameter (Nplt = 100) 

c---
c constants
c---

      null = 0

c---
c open output file
c---

      open(1,file="ann2lel0.out",status="unknown")

c---
c problem parameter specification
c---

      write (6,*) 
      write (6,*) " Enter:"
      write (6,*) 
      write (6,*) " 1 to read parameters from file: ann2lel0.dat"
      write (6,*) " 2 to type them in"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ienrd

      if(Ienrd.eq.0) stop

  97  Continue      ! move to desired location to get series of graphs

c--------------------
c read from data file
c--------------------

      if(Ienrd.eq.1) then

       open(3,file="ann2lel0.dat")

       read (3,*) eps
       read (3,*) gref
       read (3,*)
       read (3,*) a      ! interface radius
       read (3,*) ai     ! inner tube radius
       read (3,*) ae
       read (3,*) visc1  ! inner fluid viscosity
       read (3,*) visc2
       read (3,*) gm0    ! prestressed shape surface tension
       read (3,*) els

       close (3)

       gr0 = gref-eps
       gr1 = gref
       gr2 = gref+eps

       if(gr0.lt.0.0000001.or.gr1.lt.0.0000001
     +                    .or.gr2.lt.0.0000001) then
        write (6,*) " ann2lel0: inappropriate input"
        stop
       End If

c-----------
      End If    ! end of reading from data file
c-----------

c-------------------
c type in parameters
c-------------------

      if(Ienrd.eq.2) then

       write (6,*)
       write (6,*) "Enter epsilon for numerical differentiation"
       write (6,*) "in computing the coefficients of the binomial"
       write (6,*) "0.10 recommended "
       write (6,*) "0 to quit"
       write (6,*) "---------"
       read  (5,*) eps

       If(eps.eq.0) Go to 99

 92    write (6,*)
       write (6,*) "Enter the reference growth rate"
       write (6,*) "      0.456 recommended "
       write (6,*) "------------------------"
       read  (5,*) gref

       gr0 = gref-eps
       gr1 = gref
       gr2 = gref+eps

       if(gr0.lt.0.0000001.or.gr1.lt.0.0000001
     +                    .or.gr2.lt.0.0000001) then
         write (6,*) " Please try again"
         Go to 92
       End If

       write (6,*)
       write (6,*) " Enter the thread radius"
       write (6,*) " 0 to quit"
       write (6,*) " ----------"
       read  (5,*) a

       if(a.lt.0.0000001) Go to 99

       write (6,*)
       write (6,*) " Enter the radius of the inner cylinder"
       write (6,*) " 99 to quit"
       write (6,*) " ----------"
       read  (5,*) ai

       if(ai.eq.99) Go to 99

       write (6,*)
       write (6,*) " Enter the radius of the external cylinder"
       write (6,*) " 0 for infinity"
       write (6,*) " 99 to quit"
       write (6,*) " ----------"
       read  (5,*) ae

       If(ae.eq.99) Go to 99

       write (6,*)
       write (6,*) " Enter the viscosity of the inner fluid"
       write (6,*) " --------------------------------------"
       read  (5,*) visc1

       write (6,*)
       write (6,*) " Enter the viscosity of the outer fluid"
       write (6,*) " --------------------------------------"
       read  (5,*) visc2

       write (6,*)
       write (6,*) " Enter the prestress tension gamma"
       write (6,*) " 99 to quit"
       write (6,*) " ----------"
       read  (5,*) gm0

       if(gm0.eq.99) Go to 99

       write (6,*)
       write (6,*) " Enter the modulus of elasticity"
       write (6,*) " 99 to quit"
       write (6,*) " ----------"
       read  (5,*) els

       if(els.eq.99) Go to 99

c-----------
      End If    ! end of typing in parameters
c-----------

c-------------------
c set matrix size: n
c-------------------

      if(ai.lt.small.and.ae.gt.small) then
       Icase = 2
       n     = 6
      elseif(ai.gt.small.and.ae.lt.small) then
       Icase = 3 
       n     = 6
      elseif(ai.lt.small.and.ae.lt.small) then
       Icase = 4
       n     = 4
      else
       Icase = 1
       n     = 8   
      end if

c-----
c menu
c-----

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 for one evaluation"
      write (6,*) " 2 for growth rate vs ka profile"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) menu

      if(menu.eq.0) Go to 99

c---
      if(menu.eq.1) then
c---
        write (6,*)
        write (6,*) "Please choose a wave number"
        write (6,*) "---------------------------"
        read  (5,*) k

        kstart = k
        Nloop = 1   ! one value
c---
      else
c---
        kstart = 0.01
        kend   = 1.51
        kstep  = (kend-kstart)/Nplt
        Nloop  = Nplt

        write (1,101) Nloop,gm0,els
c---
      End If
c---

c--------------------------
c scan the wave number axis
c--------------------------

      k = kstart

      Do 44 Iloop = 1,Nloop

c---
c evaluate the secular matrix
c and its determinant
c---

      Idet = 1         ! compute the determinant

      call matrix
     +
     +  (gr0
     +  ,a
     +  ,ai
     +  ,ae
     +  ,visc1
     +  ,visc2
     +  ,gm0
     +  ,els
     +  ,k
     +  ,Icase
     +  ,n
     +  ,Idet
     +  ,secm
     +  ,det0
     +  ,Istop
     +  )

      if(Istop.eq.1) Go to 97

c---
c second evaluation
c---

      call matrix
     +
     +   (gr1
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,els
     +   ,k
     +   ,Icase
     +   ,n
     +   ,Idet
     +   ,secm
     +   ,det1
     +   ,Istop
     +   )

      if(Istop.eq.1) Go to 97

c---
c third evaluation
c---

      call matrix
     +
     +  (gr2
     +  ,a
     +  ,ai
     +  ,ae
     +  ,visc1
     +  ,visc2
     +  ,gm0
     +  ,els
     +  ,k
     +  ,Icase
     +  ,n
     +  ,Idet
     +  ,secm
     +  ,det2
     +  ,Istop
     +  )

      if(Istop.eq.1) Go to 97

c---
c compute binomial coefficients
c by exact finite differences
c---

      cc = det1
      bb = (det2-det0)/(2.0*eps)
      aa = (det2-2.0*det1+det0)/(2.0*eps*eps)

      disc = bb**2-4.0*aa*cc

      if(disc.lt.0) then
        write (6,*)
        write (6,*) " ann2lel0: inomial has complex roots (!?)"
        write (6,*)
        stop
      end if

      root1 = (-bb-Dsqrt(disc))/(2.0*aa)
      root2 = (-bb+Dsqrt(disc))/(2.0*aa)

c     conf1 = aa*root1**2+bb*root1+cc
c     conf2 = aa*root2**2+bb*root2+cc
c     write (6,*) conf1,conf2

      root1 = root1 + gref
      root2 = root2 + gref

c     If(k*a.gt.1.0) then
c      save = root1
c      root1 = root2
c      root2 = save
c     End If

c---
c  solve the linear system with the first root
c  and compute the x-displacement coefficient xi1
c---

      Idet = 0       ! do not compute the determinant
                     ! (system is singular and underflow may occur)
      Isym_gel = 0   ! matrix is not symmetric
      Iwlpvt   = 1   ! pivoting enabled
      nn = 4         ! size of inner-fluid expansion (normally 4 terms)
      if(ai.lt.small) nn = 2

      call matrix
     +
     +   (root1
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,els
     +   ,k
     +   ,Icase
     +   ,n
     +   ,Idet
     +   ,secm
     +   ,det
     +   ,Istop
     +   )

      if(Istop.eq.1) Go to 97

c---
c set the value of the last unknown equal to 1.0
c and solve the (n-1)x(n-1) system
c for the remaining unknowns
c---

      sol(n) = 1.0

      Do i=1,n-1
        rhs(i) = -secm(i,n)
      End Do

      call gel
     +
     +  (n-1
     +  ,secm
     +  ,rhs
     +  ,sol
     +  ,Isym_gel,Iwlpvt
c    +  ,l,u
     +  ,det
     +  ,Istop
     +  )


c---
c compute c*a_1 = ca1
c         U(sigma=a)
c and then xi_1
c---

      ca1 = 0.0
      ui  = 0.0
      Do i=1,nn
       ca1 = ca1 + sol(i)*secm(1,i)
       ui  = ui  + sol(i)*secm(2,i)
      End Do

      xi1 = -ui/(k*ca1)

c---
c  Repeat for the second root:
c  Solve the linear system with the second root
c  and compute the x-displacement coefficient xi2
c---

      call matrix
     +
     +   (root2
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,els
     +   ,k
     +   ,Icase
     +   ,n
     +   ,Idet
     +   ,secm
     +   ,det
     +   ,Istop
     +   )

      if(Istop.eq.1) Go to 97

      Do i=1,n-1
        rhs(i) = -secm(i,n)
      End Do

      call gel
     +
     +   (n-1
     +   ,secm
     +   ,rhs
     +   ,sol
     +   ,Isym_gel,Iwlpvt
c    +   ,l,u
     +   ,det
     +   ,Istop
     +   )

      ca1 = 0.0
      ui  = 0.0
      Do i=1,nn
       ca1 = ca1 + sol(i)*secm(1,i)
       ui  = ui  + sol(i)*secm(2,i)
      End Do

      xi2 = -ui/(k*ca1)

c---
c printing session
c---

      if(menu.eq.1) then  ! print unscaled values

        write (6,130) root1,root2
        write (6,131) xi1,xi2

      else

        print1 = root1/(gm0+els)    ! scale
        print2 = root2/(gm0+els)
        print3 = log10(abs(xi1))
        print4 = log10(abs(xi2))

        write (6,101) Iloop,k,print1,xi1,print2,xi2
        write (1,101) Iloop,k,print1,print3,print2,print4

      endif

      k = k+kstep

  44  Continue

c---
c return to the main menu
c---

      Go to 97

c-----
c done
c-----

  99  Continue

      write (1,101) Null
      write (1,*)
      write (1,*) a      ! interface radius
      write (1,*) ai     ! inner tube radius
      write (1,*) ae
      write (1,*) visc1  ! inner fluid viscosity
      write (1,*) visc2

      close (1)

 100  Format (1x,i3,10(1x,f20.8))
 101  Format (1x,i3,1x,f9.6,1x,f9.6,1x,f12.6,1x,f9.6,1x,f12.6)
 130  Format (" roots                 : ",2(1x,f15.10))
 131  Format (" x-displ coeff (delta) : ",2(1x,f15.10))

      Stop
      End
