       subroutine chan2l0
     +
     +  (RL          ! wave length
     +  ,h           ! channel semi-width
     +  ,y0          ! position of the unperturbed interface
     +  ,th0         ! inclination angle
     +  ,vs1,vs2     ! viscosities
     +  ,den1,den2   ! densities
     +  ,gamma
     +  ,U1,U2
     +  ,pg
     +  ,gac
     +  ,Iframe
     +  ,Ishow
     +  ,xi1,xi2
     +  ,uI
     +  ,cr,ci
     +  ,sr,si
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------------------------------
c  Instability of the two-layer channel flow.
c
c  Computation of the
c  phase velocity and growth rate
c
c  Shear-, Pressure, or Gravity-driven flow
c
c  1 refers to lower fluid, 2 to upper fluid.
c
c       ----> U2
c   =========================
c            Fluid 2
c   --------------------------
c            Fluid 1
c   =========================
c       --> U1
c
c LEGEND:
c ------
c
c  pg = -dp/dx pressure gradient for Poiseuille flow
c
c  Iframe = 0	stationary frame
c	    1	moving with the interfacial velocity
c
c  uI:  interfacial velocity
c  th0: inclination angle
c
c  gac:  acceleration due to gravity.
c
c  NOTE: ci is unaffected by the base flow,
c        only cr depends on the base flow
c-----------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension QR(4,4),QI(4,4)

      Double Precision k,ks
      Double Precision l,lc

      Parameter (tol=1.0E-06)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c-------- 
c prepare
c-------- 

      gx = gac*Dsin(th0) ! gravity vector
      gy =-gac*Dcos(th0)

      k  = pi2/RL        ! wave number
      ks = k*k

      h1 = h+y0          ! lower layer thickness
      h2 = h-y0          ! upper layer thickness
      hs = h*h

      r  = h2/h1         ! thickness ratio
      rs = r*r
      r1 = 1.0D0+r

      l = vs2/vs1       ! viscosity ratio
      lc = 1.0D0-l

      d = den2/den1     ! density ratio
      Drho  = den1-den2 ! density difference

      DU = U2-U1

      hh1 = k*h1
      hh2 = k*h2

      ch1 = Dcosh(hh1)
      ch2 = Dcosh(hh2)

      sh1 = Dsinh(hh1)
      sh2 = Dsinh(hh2)

      hh = k*h
      ch = Dcosh(hh)
      ch = 1.0D0

c----------------------------------------------
c  Shear rate one either side of the interface,
c
c  xi1, xi2
c----------------------------------------------

      xi1 = DU/h1 * l/(l+r)
     +     -pg*(l-rs)       /( vs1*r1*(l+r) )
     +     -den1*gx*(l-d*rs)/( vs1*r1*(l+r) )

      xi2 = xi1/l

c     write (6,*) " chan2l0: interface shear rates: ",xi1,xi2

c--------------------
c  Interface velocity
c--------------------

      uI =  (r*U1+l*U2)/(r+l)
     +
     +  + (pg*hs/vs1) * 2.0D0*r/(r1*(l+r)) 
     +  + (den1*gx*h*h/vs1)*
     +           (1.0D0+d*r)*2.0D0*r /(r1*r1*(l+r)) 

c     write (6,*) " chan2l0: interfacial velocity: ",uI

c----------------------------
c  compile the secular matrix
c----------------------------

      Do i=1,4
       Do j=1,4
        QR(i,j) = 0.0D0   ! real
        QI(i,j) = 0.0D0   ! imaginary
       End Do
      End Do

c--------
c prepare
c--------

      XIC1 = xi1/k
      XIC2 = xi2/k
      DXIC = XIC2-XIC1

      Box = Drho*gx/(2.0D0*vs1*ks)
      Boy = Drho*gy/(2.0D0*vs1*ks)

      PIC = gamma/(2.0D0*vs1) - Boy

      C12R = hh1*( DXIC*ch1-Box*sh1)
      C12I = PIC * (hh1*ch1-sh1)

      C22R = DXIC*(ch1+hh1*sh1)-Box * (sh1+hh1*ch1)
      C22I = PIC * hh1*sh1

c---
c real part
c---

      QR(1,1) = ch2
      QR(1,2) = hh2*ch2
      QR(1,3) = sh2
      QR(1,4) = hh2*sh2

      QR(2,1) = sh2
      QR(2,2) = ch2 + hh2*sh2
      QR(2,3) = ch2
      QR(2,4) = sh2 + hh2*ch2

      QR31    = ch1 - lc*hh1*sh1
      QR(3,1) = QR31
      QR(3,2) = -hh1*ch1
      QR(3,3) = -l*sh1 - lc*hh1*ch1
      QR(3,4) =  l*hh1*sh1

      QR41    = -sh1 + lc*(sh1+hh1*ch1)
      QR(4,1) = QR41
      QR(4,2) = ch1+hh1*sh1
      QR(4,3) = ch1 + lc*hh1*sh1
      QR(4,4) = -l*(sh1+hh1*ch1)

c--------------------
c det(Q) = a zeta + b
c
c where b = Q(0)
c--------------------

c---------
c zeta = 0
c---------

      zR = 0.0D0
      zI = 0.0D0

c-------------
c     Do i=1,4
c      Do j=1,4
c       QR(i,j) = QR(i,j)/ch
c       QI(i,j) = QI(i,j)/ch
c      End Do
c     End Do
c-------------

      call det_44c
     +
     +   (QR,QI
     +   ,DetR,DetI
     +   )

c---
      If(Ishow.eq.1) then
       Do i=1,4
         write (6,101) (QR(i,j),QI(i,j),j=1,4)
       End Do
       write (6,102) DetR,DetI
      End If
c---

c-------------
c     Do i=1,4
c      Do j=1,4
c       QR(i,j) = QR(i,j)*ch
c       QI(i,j) = QI(i,j)*ch
c      End Do
c     End Do
c-------------

      BR = DetR
      BI = DetI

c---------
c zeta = 1
c
c Note: Q(1) = a + b
c---------

      zR = 1.0D0
      zI = 0.0D0

      QR(3,1) = QR31 - (zR*C12R-zI*C12I)
      QI(3,1) =      - (zR*C12I+zI*C12R)

      QR(4,1) = QR41 + (zR*C22R-zI*C22I)
      QI(4,1) =        (zR*C22I+zI*C22R)

c-------------
c     Do i=1,4
c      Do j=1,4
c       QR(i,j) = QR(i,j)/ch
c       QI(i,j) = QI(i,j)/ch
c      End Do
c     End Do
c-------------

      call det_44c
     +
     +   (QR,QI
     +   ,detR,detI
     +   )

c---
      If(Ishow.eq.1) then
        Do i=1,4
          write (6,101) (QR(i,j),QI(i,j),j=1,4)
        End Do
       write (6,102) detR, detI
      End If
c---

c-------------
c     Do i=1,4
c      Do j=1,4
c       QR(i,j) = QR(i,j)*ch
c       QI(i,j) = QI(i,j)*ch
c      End Do
c     End Do
c-------------

      AR = detR - BR
      AI = detI - BI

c------------------
c root: zeta = -b/a
c------------------

      Dnm = AR*AR+AI*AI

      zR = -(BR*AR + BI*AI)/Dnm
      zI =  (BR*AI - BI*AR)/Dnm

c     write (6,103) zR,zI

c---------------------------------------
c verify
c---

      QR(3,1) = QR31 - (zR*C12R-zI*C12I)
      QI(3,1) =      - (zR*C12I+zI*C12R)

      QR(4,1) = QR41 + (zR*C22R-zI*C22I)
      QI(4,1) =        (zR*C22I+zI*C22R)

c-------------
c     Do i=1,4
c      Do j=1,4
c       QR(i,j) = QR(i,j)/ch
c       QI(i,j) = QI(i,j)/ch
c      End Do
c     End Do
c-------------

      call det_44c
     +
     +   (QR,QI
     +   ,detR,detI
     +   )

c-------------
c     Do i=1,4
c      Do j=1,4
c       QR(i,j) = QR(i,j)*ch
c       QI(i,j) = QI(i,j)*ch
c      End Do
c     End Do
c-------------

c     If(Dabs(detR).gt.tol.or.Dabs(detI).gt.tol) then
c       write (6,*)  " chan2l0: root finder failed"
c       write (6,102) detR, detI
c     End If

c---
c End of verifying
c---------------------------------------

c-------------------------------------
c complex phase velocity: c = uI + 1/z
c-------------------------------------

      Dnm = ZR*ZR+ZI*ZI

      cr =  ZR/Dnm
      ci =- ZI/Dnm

      If(Iframe.eq.0) cr = cr+uI

c--------------------
c complex growth rate
c--------------------

      sr = k*cr
      si = k*ci

c-----
c Done
c-----

  101 Format (1x,4(3x,f13.5,1x,f13.5))
  102 Format ("Determinant:",1x,f15.10,1x,f15.10)
  103 Format ("zeta       :",1x,f15.10,1x,f15.10)

      Return 
      End
