      subroutine prony 
     +
     +   (Npoints
     +   ,signal
     +   ,Dt
     +   ,Nexp
     +   ,Iopt     ! 1 for pure exponentials, 2 to add a constant
     +   ,z
     +   ,sigma
     +   ,coeff
     +   )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------------
c Prony fitting of a signal consisting
c of a sum of complex exponentials
c
c SYMBOLS:
c -------
c
c Npoints:  Number of points in time series
c
c signal:   Time series of the signal
c
c Dt:	    Time step
c
c Nexp:   Number of complex modes.
c         A complex mode (CM) is a combination:
c
c     CM = [ cr*cos(sr*t) + ci*sin(sr*t) ] * exp(si*t)
c
c M:  size of the overdetermined system
c
c z:  z(j,1) is real value of exp(-i sigma Dt)
c     z(j,2) is imag value of exp(-i sigma Dt)
c
c sigma:  sigma(j,1) is real part of sigma
c         sigma(j,2) is imag ovalue rt f sigma
c
c pol_cf: coefficients of the characteristic polynomial
c
c--------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension signal(32000)

      Dimension am(32000,32),rhs(32000)

      Dimension am_n(32,32),rhs_n(32),sol(32)

      Dimension z(20,2),sigma(20,2),coeff(20,2)
      Dimension pol_cf(20)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c--------
c prepare
c--------

      Nexp2 = 2*Nexp      ! number of complex exponentials
                          ! including the complex conjugates

      M = Npoints-Nexp2   ! see legend

c--------------------------------------
c form the over-determined Prony system
c--------------------------------------

      Do i=1,M                    ! pure exponential fitting
        Do j=1,Nexp2
         am(i,j) = signal(i+j-1)
        End Do
        rhs(i) = signal(i+Nexp2)
      End Do

c----------------------------
c Subtract adjacent equations
c----------------------------

      If(Iopt.eq.2) then  ! exponential fitting with a constant

      Do i=1,M-1
        Do j=1,Nexp2
         am(i,j) = am(i,j) -am(i+1,j)
        End Do
        rhs(i) = rhs(i)-rhs(i+1)
      End Do

      M = M-1

      End If

c-----------------
c Printing session
c-----------------

      If(M.lt.Nexp2) then
        write (6,*) "sorry, not enough points in the time series"
        stop
      End If

c-----------------
c Printing session
c-----------------

c     write (6,*)
c     write (6,*) " prony: over-determined system of M equations:"
c     write (6,*)
c
c     Do i=1,m
c      write (6,100) (am(i,j),j=1,Nexp2),rhs(i)
c     End Do

c------------------------------------------------
c Formulate the normal system of 2*Nexp equations
c------------------------------------------------

      Do i=1,Nexp2

        Do j=1,Nexp2
          am_n(i,j) = 0.0D0
          Do l=1,M
           am_n(i,j) = am_n(i,j)+am(l,i)*am(l,j)
          End Do
        End Do

        rhs_n(i) = 0.0D0
        Do l=1,M
         rhs_n(i) = rhs_n(i)+am(l,i)*rhs(l)
        End Do

      End Do

c--------------------------------------------------
c Solve the normal system by Cholesky decomposition
c--------------------------------------------------

c---------------
c     write (6,*)
c     write (6,*) " Solving for the polynomial coefficients"
c     write (6,*)
c---------------

      Isee = 0   ! do not display

      call chol_c 
     +
     +   (Nexp2
     +   ,am_n
     +   ,rhs_n
     +   ,Isee
     +   ,sol
     +   )

c---------------
c     write (6,*)
c     write (6,*) " Normal system and solution"
c     write (6,*)
c     Do i=1,Nexp2
c       write (6,103) (am_n(i,j),j=1,Nexp2),rhs_n(i),sol(i)
c     End Do
c---------------

c---
c Compute the polynomial coefficients
c---

      pol_cf(1) = 1.0D0

      If(Nexp.eq.1) then

       pol_cf(2) =-sol(2)
       pol_cf(3) =-sol(1)

      Else If(Nexp.eq.2) then

       pol_cf(2) =-sol(4)
       pol_cf(3) =-sol(3)
       pol_cf(4) =-sol(2)
       pol_cf(5) =-sol(1)

      Else If(Nexp.eq.3) then

       pol_cf(2) =-sol(6)
       pol_cf(3) =-sol(5)
       pol_cf(4) =-sol(4)
       pol_cf(5) =-sol(3)
       pol_cf(6) =-sol(2)
       pol_cf(7) =-sol(1)

      Else If(Nexp.eq.4) then

       pol_cf(2) =-sol(8)
       pol_cf(3) =-sol(7)
       pol_cf(4) =-sol(6)
       pol_cf(5) =-sol(5)
       pol_cf(6) =-sol(4)
       pol_cf(7) =-sol(3)
       pol_cf(8) =-sol(2)
       pol_cf(9) =-sol(1)

      Else

       write (6,*) "prony: sorry, too many exponentials"
       write (6,*)
       write (6,*) "Please modify prony.f"
       stop

      End If

c---------
c printing:
c---------

       write (6,*)
       write (6,*) "prony: polynomial coefficients:"
       write (6,*)

       Do i=1,2*Nexp+1
        write (6,*) i, pol_cf(i)
       End Do

c---------
c find z's 
c using bairstow's method
c---------

      call bairstow 
     +
     +  (pol_cf
     +  ,Nexp2
     +  ,z
     +  ,Iflag
     +  )

      If(Iflag.eq.0) then
       write (6,*)
       write (6,*) " Polynomial root-finder failed"
       stop
      End If

c---------------------
c recover sigma's
c z = exp(-i*sigma*Dt)
c---------------------

      Do i=1,Nexp2
        tr = z(i,1)
        ti = z(i,2)
        rd = Dsqrt(tr**2+ti**2)
        sigma(i,2) = log(rd)/Dt   ! imaginary part
        ph = Dacos(tr/rd)
        If(ti.lt.0) ph = -ph
        sigma(i,1) = -ph/Dt       ! real part
      End Do

c----------------------------------------------
c  Form the over-determined system for the
c  coefficients of the exponentials
c
c  Coefficients correspond to sigma(1), sigma(3), ...
c
c  The coefficients corresponding to sigma(2), sigma(4), ...
c  arise by switching the sign of the imaginary part
c----------------------------------------------

      Do i=1,Npoints

       time = Dt*(i-1.0D0)
       Ic = 0

       Do j=1,Nexp2,2
        expp  = Dexp(sigma(j,2)*time)
        angle =      sigma(j,1)*time
        Ic = Ic+1
        am(i,ic) = expp*Dcos(angle)
        Ic = Ic+1
        am(i,Ic) = expp*Dsin(angle)
       End Do
       rhs(i) = signal(i)

      End Do

c---------------
c     write (6,*)
c     Do i=1,Npoints
c      write (6,100) (am(i,j),j=1,Nexp2),rhs(i)
c     End Do
c---------------

c-------------------------------------------
c Form the normal system of 2*Nexp equations
c-------------------------------------------

      Do i=1,Nexp2

        Do j=1,Nexp2
         am_n(i,j) = 0.0D0
         Do l=1,Npoints
          am_n(i,j) = am_n(i,j)+am(l,i)*am(l,j)
         End Do
        End Do

        rhs_n(i) = 0.0D0
        Do l=1,Npoints
         rhs_n(i) = rhs_n(i)+am(l,i)*rhs(l)
        End Do

      End Do

c--------------------------------------------------
c Solve the normal system by Cholesky decomposition
c--------------------------------------------------

c     write (6,*)
c     write (6,*) " Solving for the coefficients of the exponentials"
c     write (6,*)

      Isee = 0   ! do not display

      call chol_c 
     +
     +  (Nexp2
     +  ,am_n
     +  ,rhs_n
     +  ,Isee
     +  ,sol
     +  )

c     Do i=1,Nexp2
c       write (6,103) (am_n(i,j),j=1,Nexp2),rhs_n(i),sol(i)
c     End Do

c--------------------
c Assign the solution
c--------------------

      Ic = 0  ! counter

      Do i=1,Nexp
       j1 = 2*i-1
       j2 = 2*i
       ic = Ic+1
       coeff(j1,1) = sol(ic)
       Ic = Ic+1
       coeff(j1,2) = sol(Ic)
       coeff(j2,1) =  coeff(j1,1) 
       coeff(j2,2) = -coeff(j1,2) 
      End Do

c-----
c Done
c-----

 100  Format (1x,f12.7,10(1x,f12.7,1x))
 103  Format (10(1x,f15.6))

      Return
      End
