      program vl_2d

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c--------------------------------------------
c Dynamical simulation of the motion of a number of
c adjacent vortex layers with constant vorticity
c separating two uniform streams,
c by the method of contour dynamics
c
c The flow is periodic in the x direction with 
c period RL
c
c The contours are described by a set of
c marker points whose position is advanced by
c the second-order Runge-Kutta method.
c
c SYMBOLS:
c -------
c
c Variables ending with g are global
c
c For example:
c
c  xg(i,l): x-position of ith point on lth contour
c  yg(i,l): y-position of ith point on lth contour
c
c CAPACITY:
c --------
c
c Maximum number of contours: 25
c Maximum number of points over each contour: 200
c------------------------------------------------

      Implicit Double Precision (a-h,o-z)

c---
c global variables
c---

      Dimension xg(0:200,25),yg(0:200,25)
      Dimension ug(0:200,25),vg(0:200,25)

      Dimension XCg  (200,25),YCg (200,25),Rg  (200,25),Sg(0:200,25)
      Dimension TH1g (200,25),TH2g(200,25),TH3g(200,25)
      Dimension orntg(200,25)

c--- related to second-order Runge-Kutta (RK2)
c--- sv stands for save

      Dimension Xsv(200,25),Ysv(200,25)
      Dimension Usv(200,25),Vsv(200,25)

c--- contour and layer properties

      Dimension NSG  (25),NSG1(25),NSG2(25)
      Dimension omega(0:25)

      Dimension ypos0(25),amp0(25),shift0(25)
      Dimension peri0(25),epif0(25)

c---
c local variables
c---

      Dimension X(0:200),Y(0:200)
      Dimension U(0:200),V(0:200)
      Dimension unused1(0:200),unused2(0:200)
      Dimension unused3(0:200),unused4(0:200)
      Dimension unused5(0:200)

      Dimension XC (200),YC (200),R  (200),S   (200)
      Dimension TH1(200),TH2(200),TH3(200),ORNT(200)

c---
c  related to time stepping
c---

      Dimension time(0:2000)

      Dimension peri(0:2000,25),epif(0:2000,25)

c---
c  various
c---

      Dimension ZZ(20),WW(20)      ! Gauss-Legendre weights

c---
c common blocks for global variables
c---

      common/XXYYg/Xg,Yg
      common/ARCCg/XCg,YCg,Rg,Sg,TH1g,TH2g,TH3g,ORNTg
      common/UUVVg/Ug,Vg

c---
c common blocks for local variables
c---

      common/XXYY/X,Y
      common/ARCC/XC,YC,R,S,TH1,TH2,TH3,ORNT
      common/UUVV/U,V

c---
c various common blocks
c---

      common/ancI1/Ncntr,NSG,NSG1,NSG2
      common/ancI3/NGL
      common/ancR1/omega
      common/ancR5/RL,wn

      common/ZZWW/ZZ,WW
      common/piii/pi,pih,piq,pi2,pi4,pi6,pi8,srpi

c---
c constants
c---

      pi   = 3.14159 265358 979 32384
      pih  = 0.5 * pi
      piq  = 0.25* pi
      pi2  = 2.0 * pi
      pi4  = 4.0 * pi
      pi6  = 6.0 * pi
      pi8  = 8.0 * pi
      srpi = sqrt(pi)

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c----------------------
c Simulation parameters
c----------------------

      write (6,*)
      write (6,*) ' Type 0 to enter type in parameters'
      write (6,*) '      1 to read them from input file: vl_2d.dat'
      write (6,*)
      write (6,*) '  Warning: if you type 0 will spend a lot'
      write (6,*) '           of time typing '
      write (6,*) ' ---------------------------------------'
      read  (5,*)   Ienrd

c------------------------
      If(Ienrd.eq.0) then
c------------------------

      write (6,*)
      write (6,*) ' Enter the number of vortex layers'
      write (6,*)
      write (6,*) ' Number of contours is one more than'
      write (6,*) ' the number of layers'
      write (6,*) ' --------------------'
      read  (5,*) Nlayers

      Ncntr = Nlayers+1

      write (6,*)
      write (6,*) ' Enter the vorticity of the layers'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' -------------'
      read  (5,*)  (omega(i),i=1,Nlayers)

      write (6,*)
      write (6,*) ' Enter the period L '
      write (6,*) ' -------------------'
      read  (5,*) RL

      write (6,*)
      write (6,*) ' Computation of the contour integral'
      write (6,*)
      write (6,*) ' Enter the number of Gauss integration points'
      write (6,*) ' over each arc'
      write (6,*)
      write (6,*) ' Choose from 2,6,12,20; default is 20'
      write (6,*) ' ------------------------------------'
      read  (5,*)   NGL

      write (6,*)
      write (6,*) ' Adaptive point redistribution'
      write (6,*)
      write (6,*) ' Enter the amximum angle subtended by an arc'
      write (6,*) ' in multiples of pi'
      write (6,*) ' ------------------'
      read  (5,*)   thmax

      write (6,*)
      write (6,*) ' Enter the minimum and maximum marker point'
      write (6,*) '       separation' 
      write (6,*) ' ------------------------------------------'
      read  (5,*)   SPmin,SPmax

      write (6,*)
      write (6,*) ' Enter thresholds for period redefinition'
      write (6,*) ' in multiples of the period L'
      write (6,*) ' ----------------------------'
      read  (5,*)   thrs1,thrs2

      write (6,*)
      write (6,*) ' Normalize layer areas after each step ?'
      write (6,*)
      write (6,*) ' Enter 0 for no, 1 for yes'
      write (6,*) ' -------------------------'
      read  (5,*)   Norm

c------------------
c Initial condition
c------------------

      write (6,*) 
      write (6,*) ' Enter the contour y position '
      write (6,*) '                   amplitude '
      write (6,*) '                   phase'
      write (6,*) ' Bottom to top'
      write (6,*) ' -------------'

      Do i = 1,Ncntr
       write (6,*) "Contour: ",i
       read  (5,*) ypos0(i),amp0(i),shift0(i)
      End Do

      write (6,*) 
      write (6,*) ' Enter the layer vorticities'
      write (6,*) 
      write (6,*) ' Bottom to top'
      write (6,*) ' ----------------------------'

      Do i = 1,Nlayers
       write (6,*) "Layer: ",i
       read  (5,*) omega(i)
      End Do

      write (6,*) 
      write (6,*) ' Enter number of points around each contour'
      write (6,*) '                             Maximum is 1000'
      write (6,*) ' Bottom to top'
      write (6,*) ' -------------------------------------------'

      Do i = 1,Ncntr
       write (6,*) "Contour: ",i
       read (5,*) NSG(i)
      End Do

c----
c Related to time steppping
c---

      write (6,*)
      write (6,*) ' After how many steps get a printout ?'
      write (6,*) ' -------------------------------------'
      read  (5,*)   Nprint

      write (6,*)
      write (6,*) ' Time integration by the RK2'
      write (6,*)
      write (6,*) ' Enter the time step Dt'
      write (6,*) ' ----------------------'
      read  (5,*)   Dt

c-----------------------------
      Else If(Ienrd.eq.1) then
c-----------------------------

      open (3,file='vl_2d.dat')

        read (3,*) RL
        read (3,*) NGL
        read (3,*)
        read (3,*) THmax
        read (3,*) SPmin
        read (3,*) SPmax
        read (3,*) thrs1,thrs2
        read (3,*)
        read (3,*) Iread
        read (3,*)
        read (3,*) Norm
        read (3,*) Nprint
        read (3,*)
        read (3,*) Dt
        read (3,*)
        read (3,*) Nlayers
        read (3,*)  

        Ncntr = Nlayers+1

        Do i=1,Ncntr
         read (3,*) idle,ypos0(i),amp0(i),shift0(i)
        End Do

c---
c If Index = 0
c properties are uniform
c---

        read (3,*)  
        read (3,*) Index,omega_ref 

        Do i=1,Nlayers
          omega(i) = omega_ref
          If(Index.ne.0) read (3,*) idle,omega(i)
          write (6,*) " Vorticity of layer ",i, " is ",omega(i)
        End Do

        read (3,*)  
        read (3,*) Index,nsg_ref   ! points around contours

        Do i=1,Ncntr
          nsg(i) = nsg_ref
          If(Index.ne.0) read (3,*) idle,nsg(i)
        End Do

c-----------
      End If   ! end of reading properties
c-----------

c--------
c prepare
c--------

      Do j=1,Ncntr
       shift0(j) = shift0(j)*pi2    ! scale the shift
      End Do

      thmax = thmax*pi
      thrs1 = thrs1*RL
      thrs2 = thrs2*RL

      omega(0)     = 0.0     ! voriticity of semi-infinite area below
      omega(Ncntr) = 0.0     ! vorticity of semi-infinite area above

      wn = pi2/RL          ! wave number

      call gauss_leg (NGL,ZZ,WW)

c---
c Initial contour perimeters and areas
c between the x axis and the contours
c---

      Do i=1,Ncntr
        peri0(i) = pi2
        epif0(i) = RL*ypos0(i)
      End Do

      Do i=1,Ncntr-1
      End Do

c---
c Displaying on screen
c---

      write (6,*)
      write (6,108)
      write (6,*)

      Do j=1,Ncntr
       write (6,107) j,ypos0(j),amp0(j),shift0(j)
      End Do

c------------------------
c Generate initial shapes
c------------------------

      If(Iread.eq.0) then

       time(1) = 0.0

       Do j = 1,Ncntr     !--------- over contours

        write (6,*)
        write (6,*) " Generating contour:",j
        write (6,*)

        NSG1(j) = NSG(j)+1
        NSG2(j) = NSG(j)+2
        step    = RL/(NSG1(j)-1.0)

        Do i = 0,NSG2(j)
          xg(i,j) = (i-1.0)*step
          tmp = wn*xg(i,j)+shift0(j)
          yg(i,j) = ypos0(j)+amp0(j)*cos(tmp)
        End Do

      End Do

c----------------------------------
c Read position of marker points
c      from file: vl_2d.inp
c-----------------------------------

      Else If(Iread.eq.1) then

        open (2,file='vl_2d.inp')

        write (6,*) 

        Do j = 1,Ncntr              !------ loop over contours

          read (2,*) NSG1(j),idle,time(1)

          write (6,*) "Reading contour ",j

            Do i = 1,NSG1(j)
              read(2,*) idle,xg(i,j),yg(i,j)
            End Do

          NSG (j) = NSG1(j)-1
          NSG2(j) = NSG (j)+2

        End Do             

        close (2)

c---
      End If             !------------ End of Iread
c---

c-------
c extend
c-------

      Do j=1,Ncntr

        Xg(0      ,j) = Xg(NSG(j),j)-RL
        Yg(0      ,j) = Yg(NSG(j),j)
        Xg(NSG1(j),j) = Xg(1,j)+RL
        Yg(NSG1(j),j) = Yg(1,j)
        Xg(NSG2(j),j) = Xg(2,j)+RL
        Yg(NSG2(j),j) = Yg(2,j)

      End Do

c----------------------
c Display initial shape
c on the screen
c----------------------

       write (6,*)
       write (6,*)  " Initial profiles"
       write (6,*)

       Do j = 1,Ncntr

        write (6,*) j

        Do i = 0,NSG2(j)
          write (6,100) j,i,xg(i,j),yg(i,j)
        End Do

       End Do

c---
c prepare output files
c---

      open (1,file='vl_2d.out')   ! for profiles
      open (4,file='vl_2d.diag')  ! for diagnostics

      write (1,*) RL

c--
c Prepare to run
c---

      read (3,*)  

      Kstep = 1    ! Global step counter

c===========================
c  TIME STEPPING BEGINS HERE    
c===========================

  90  Continue

      If(Ienrd.eq.0) then
        write (6,*)
        write (6,*) 'Enter number of steps before pausing'
        write (6,*) '0 to quit'
        write (6,*) '---------'
        read  (5,*)  Nstep
      Else
        read  (3,*)  Nstep
      End If

      If(Nstep.eq.0) Go to 99

c---
c initialize
c---

      Istep  = 1         ! Batch step counter

      Iprint = Nprint    ! Counter for printing

  97  Continue

      write (6,111) Istep,Nstep

c-----------------------------
c Check the point distribution
c Compute properties of arcs
c-----------------------------

      Do j = 1,Ncntr                !----- over contours

      write (6,*) " vl_2d: checking contour:",j

c---
c transfer into local variables
c---

      Do i = 0,NSG2(j)
       x(i) = xg(i,j)
       y(i) = yg(i,j)
      End Do

      ICH1 = 1
      ICH2 = 1
      ICH3 = 1
      ICH4 = 1

        call prd_2d_pr
     +
     +   (NSG(j)
     +   ,RL
     +   ,ICH1,THMAX
     +   ,ICH2,SPMAX
     +   ,ICH3,SPMIN
     +   ,ICH4,thrs1,thrs2
     +   ,peri(Kstep,j),epif(Kstep,j)
     +   ,unused10,unused11
     +   ,unused1,unused2,unused3,unused4,unused5
     +   ,Istop
     +   )

        If(Istop.eq.1) Go to 99

        NSG1(j) = NSG(j)+1
        NSG2(j) = NSG(j)+2

        write (6,*) " vl_2d: done"

c---------------------
c Normalize layer areas
c---------------------

      If(Norm.eq.1.and.abs(epif0(j)).gt.0.00000001) then

        write (6,*) " Normalize area of layer : ",j

        scale = epif0(j)/epif(Kstep,j)

        Do i = 0,NSG2(j)
         y(i) = y(i)*scale
        End Do

        ICH1 = 0   ! redistribution disabled
        ICH2 = 0
        ICH3 = 0
        ICH4 = 0

        call prd_2d_pr
     +
     +    (NSG(j)
     +    ,RL
     +    ,ICH1,THMAX
     +    ,ICH2,SPMAX
     +    ,ICH3,SPMIN
     +    ,ICH4,thrs1,thrs2
     +    ,peri(Kstep,j),epif(Kstep,j)
     +    ,unused10,unused11
     +    ,unused1,unused2,unused3,unused4,unused5
     +    ,Istop
     +    )

        If(Istop.eq.1) Go to 99

      End If

c---
c transfer into global variables
c---

      Do i = 1,NSG1(j)
            xg(i,j) =    x(i)
            yg(i,j) =    y(i)
           xcg(i,j) =   xc(i)
           ycg(i,j) =   yc(i)
            rg(i,j) =    r(i)
            sg(i,j) =    s(i)
          th1g(i,j) =  th1(i)
          th2g(i,j) =  th2(i)
          th3g(i,j) =  th3(i)
         orntg(i,j) = ornt(i)
      End Do

c---
c extend
c---

      xg(0,j) = xg(NSG(j),j)-RL
      yg(0,j) = yg(NSG(j),j)
      sg(0,j) = sg(NSG(j),j)-sg(NSG1(j),j)

      it = NSG1(j)
      xg(it,j) = xg(1,j)+RL
      yg   (it,j) = yg(1,j)

      it = NSG2(j)
      xg(it,j) = xg(2,j)+RL
      yg(it,j) = yg(2,j)
      sg(it,j) = sg(2,j)+sg   (NSG1(j),j)

c---
c scale contour arc length and area
c between contour and the x axis
c by the initial values
c---

      peri(Kstep,j) = peri(Kstep,j)/peri0(j)

      If(abs(epif0(j)).gt.0.00000001)
     +   epif(Kstep,j) = epif(Kstep,j)/epif0(j)


      End Do                !------ loop over layers

c-----------------
c Printing session
c-----------------

      write (6,*)
      write (6,113) time(Kstep),Kstep
      write (6,*)
      write (6,305) (peri(Kstep,j),j=1,Ncntr)
      write (6,306) (epif(Kstep,j),j=1,Ncntr)

      write (4,113) time(Kstep),Kstep
      Do j=1,Ncntr
       write (4,104) j,peri(Kstep,j), epif(Kstep,j)
      End Do

c-----------------
c compute the velocity
c-----------------

      call vl_2d_vel (Istop)

      if(Istop.eq.1) Go to 99

c---
c printing session
c
c Print marker point position and velocities
c---

      If(Iprint.eq.Nprint) then

       Do j=1,Ncntr
        write (1,103) NSG1(j),j,time(Kstep),Dt
          Do i=1,NSG1(j)
            write (1,120) i,Xg(i,j),Yg(i,j)
c    +                     ,Ug(i,j),Vg(i,j)
          End Do
       End Do

       Iprint = 0

      End If

c---
c save
c---

      Do j=1,Ncntr
        Do i=1,NSG1(j)
         Xsv(i,j) = Xg(i,j)
         Ysv(i,j) = Yg(i,j)
         Usv(i,j) = Ug(i,j)
         Vsv(i,j) = Vg(i,j)
          Xg(i,j) = Xg(i,j) + Dt*Ug(i,j)
          Yg(i,j) = Yg(i,j) + Dt*Vg(i,j)
        End Do
      End Do

c---
c second step in RK2
c---

      ICH1 = 0
      ICH2 = 0
      ICH3 = 0

      Do j=1,Ncntr      ! loop over contours

c---
c extend
c---

         Xg(0,j) = Xg(NSG(j),j)
         Yg(0,j) = Yg(NSG(j),j)

      Xg(NSG1(j),j) = Xg(1,j)
      Yg(NSG1(j),j) = Yg(1,j)

      Xg(NSG2(j),j) = Xg(2,j)
      Yg(NSG2(j),j) = Yg(2,j)

c---
c Transfer into prd
c---

      Do i=0,NSG2(j)
        x(i) = xg(i,j)
        y(i) = yg(i,j)
      End Do

c------------------------------
c Geometry at intermediate step
c------------------------------

        call prd_2d_pr
     +
     +    (NSG(j)
     +    ,RL
     +    ,ICH1,THMAX
     +    ,ICH2,SPMAX
     +    ,ICH3,SPMIN
     +    ,ICH4,thrs1,thrs2
     +    ,peri,epif,xcen,ycen
     +    ,unused1,unused2,unused3,unused4,unused5
     +    ,Istop
     +    )

      If(Istop.eq.1) Go to 99

c---
c transfer from prd
c---

        Do i=1,NSG1(j)
            xg(i,j) =    x(i)
            yg(i,j) =    y(i)
           xcg(i,j) =   xc(i)
           ycg(i,j) =   yc(i)
            rg(i,j) =    r(i)
            sg(i,j) =    s(i)
          th1g(i,j) =  th1(i)
          th2g(i,j) =  th2(i)
          th3g(i,j) =  th3(i)
         orntg(i,j) = ornt(i)
      End Do

c---
      End Do                    ! End of loop over drops
c---

c---------------------------
c Second velocity evaluation
c---------------------------

      call vl_2d_vel (Istop)

      If(Istop.eq.1) Go to 99

c---
c second step in RK2
c---

      Dth = 0.5*Dt

      Do j=1,Ncntr
        Do i=1,NSG(j)
         Xg(i,j) = Xsv(i,j)+Dth*(Usv(i,j)+Ug(i,j))
         Yg(i,j) = Ysv(i,j)+Dth*(Vsv(i,j)+Vg(i,j))
        End Do
      End Do

c---
c extend
c---

      Do j=1,Ncntr

      Xg(0,j) = Xg(NSG(j),j)-RL
      Yg(0,j) = Yg(NSG(j),j)

         Xg(NSG1(j),j) = Xg(1,j)+RL
         Yg(NSG1(j),j) = Yg(1,j)

      Xg(NSG2(j),j) = Xg(2,j)+RL
      Yg(NSG2(j),j) = Yg(2,j)

      End Do

c---
c reset counters and time
c---

      Istep  = Istep+1
      Kstep  = Kstep+1
      Iprint = Iprint+1

      time(Kstep) = time(Kstep-1) + Dt

      If(Istep.le.Nstep) Go to 97

      Go to 90

c-.-.-.-.-.-.-.-.-.-.
c SIMULATION HAS ENDED
c-.-.-.-.-.-.-.-.-.-.

c----------------------------------
c Finish up by printing final state
c----------------------------------

  99  Continue

c---
c print profiles in a restart file
c---

       open (9,file="vl_2d.rst")    ! restart file

       Do j=1,Ncntr

        write (1,103) NSG1(j),j,time(Kstep)
        write (9,103) NSG1(j),j,time(Kstep)

          Do i = 1,NSG1(j)
            write (1,120) i,Xg(i,j),Yg(i,j)
c    +                     ,Ug(i,j),Vg(i,j)
            write (9,120) i,Xg(i,j),Yg(i,j)
c    +                     ,Ug(i,j),Vg(i,j)
          End Do

       End Do

       write (1,112) Null
       write (9,112) Null

      close (9)

c---
c Print run parameters
c---

      thmax = thmax/pi  ! unscale
      thrs1 = thrs1/RL  ! unscale
      thrs2 = thrs2/RL  ! unscale

      write (1,105)  Ncntr
     +              ,NGL
     +              ,THmax,SPmax,SPmin
     +              ,RL
     +              ,Iread
     +              ,Norm
     +              ,Nprint
     +              ,Dt

      write (1,*)
      write (1,106)
      write (1,*)

      Do j=1,Ncntr
       write (1,107) j,omega(j)
      End Do

      write (1,*)
      write (1,108)
      write (1,*)

      Do j=1,Ncntr
       write (1,107) j,ypos0(j),amp0(j),shift0(j)
      End Do

c---
c close down
c---

      write (1,*)
      write (1,110)
      write (4,102) Null

      close (1)
      If(Ienrd.eq.1) close(3)

 100  Format (1X,I2,1X,I2,10(1X,F10.5))
 102  Format (1X,I4,5(1X,F12.8))
 120  Format (1X,I4,2(1X,F12.8),2(1X,F8.4),2(1X,F12.8))
 103  Format (1X,I2,1X,I2,3(1X,F15.5))
 104  Format (1X,I4,20(1X,F9.5))
 105  Format (/,
     +        ' Ncntr  = ',I2,/,/,
     +        ' NGL    = ',I1,/,
     +        ' THMAX  = ',F6.3,/,
     +        ' SPMAX  = ',F6.3,/,
     +        ' SPMIN  = ',F6.3,/,
     +        ' RL     = ',F7.4,/,
     +        ' Iread  = ',I1,/,
     +        ' Norm   = ',I1,/,
     +        ' Nprint = ',I3,/,
     +        ' Dt     = ',F7.4,/
     +        )


 106  Format ('omega')
 107  Format (1x,i2,10(1x,f10.5))
 108  Format ('y_position, amplitude, shift')

 109  Format (' T=',F7.3,' S=',F10.7,' A=',F10.7,
     +        ' X=',F8.5,' Y=',F8.5)
 110  Format ( " program: vl_2d",/,/)
 111  Format ( " Executing step ",I4, " Total steps: ",I4)
 112  Format (1X,I4,1X,I4)
 113  Format (1X,F8.5,1X,I6)
 115  Format (10(1X,f10.5))

 140  Format (1X,I4,1x,f10.6,20(1X,F8.5))
 145  Format (1X,I4,1x,f15.6,2(1X,F8.5),1X,f15.5)

 169  Format (1X,F8.4,100(1x,f7.5))

 305  Format (' vl_2d: contour arc length:',10(1x,10(1x,F6.3)))
 306  Format (' vl_2d: surface area      :',10(1x,10(1x,F6.3)))

      Stop
      End
