function kp_cc

%=============
% Karman-Pohlhousen method for flow
% past a circular cylinder
%=============

%---
close all
clear all
%---

%---
% parameters
%---

a = 1.0; % cylinder radius
U0 = 1.0; % incident velocity
nu = 1.0; % kinematic viscosity
mu = 1.0; % kinematic viscosity
Ndiv = 2*128;  % divisions
angle = 0.60; % maximum integration angle
              % in multiples of pi
%---
% prepare
%---

Dtht = angle*pi/Ndiv;
Dx   = a*Dtht;       % differential arc length

%---
% initial values
%---

 tht = 0.0;
 lam = 0.0770;
 Lam = 7.052;
 [lam,Lam] = Laminit(Lam)
 Up  = 2.0*U0*cos(tht)/a;
 z   = lam/Up;

%---
% integrate
%---

 for i=1:Ndiv

  U   = 2.0*U0*sin(tht);     % velocity
  Up  = 2.0*U0*cos(tht)/a;   % acceleration
  tmp = 37.0-Lam/3.0-5.0*Lam*Lam/144.0;
  H     = 31.5*(3.0-Lam/12.0)/tmp;    % (10.6.21)
  S     = (2.0+Lam/6.0)*tmp/315.0;     % (10.6.21)
  Del  = sqrt(abs(nu*Lam/Up));    % (10.6.13)
  del  = Del*(3.0-Lam/12.0)/10.0;    % (10.6.16)
  sw    = mu*U/Del*(2.0+Lam/6.0);   % (10.6.16)

  THeta = 0.0;
  if(abs(sw)>0.0000001)
    THeta =  mu*U*S/sw;   % (10.6.20)
  end

  xplot(i)=tht/pi;
  Lambda(i)=Lam/12.0;
  lambda(i)=lam;
  delta(i)=del;
  Delta(i)=Del;
  Theta(i)=THeta;
  swall(i)=sw;
  shape(i)=S;
  shear(i)=H;

  if(i==1)
    fv = 0.0;
  else
    Lam = root(lam,Lam);    % solve  for Lambda
    fv = phase_vel (U0,tht,Lam,lam);
  end

  zsave = z;
  fvsave = fv;
  tht = tht+Dtht;
  z   = z + fv*Dx;
  Up  = 2.0*U0*cos(tht)/a;
  lam = z*Up;
  Lam = root(lam,Lam);    % solve  for Lambda
  fv = phase_vel (U0,tht,Lam,lam);
  z = zsave + 0.5*(fv+fvsave)*Dx;
  lam = z*Up;

end

figure(1)
hold on
plot(xplot,Lambda,'-k')
plot(xplot,lambda,'--k')
plot([0 0.6],[-1 -1])
xlabel('\theta/\pi','fontsize',15)
ylabel('\Lambda/12,        \lambda','fontsize',15)
set(gca,'fontsize',15)
axis([0 angle -1 1])
box

figure(2)
hold on
plot(xplot,Delta,'k-')
plot(xplot,delta,'k--')
plot(xplot,Theta,'k:')
xlabel('\theta/\pi','fontsize',15)
ylabel('\Delta,   \delta,   \Theta','fontsize',15)
set(gca,'fontsize',15)
axis([0 angle 0 5])
box

figure(3)
hold on
plot(xplot,swall,'k-')
plot(xplot,shape,'k--')
xlabel('\theta/\pi','fontsize',15)
ylabel('\sigma_{xy},       S','fontsize',15)
set(gca,'fontsize',15)
axis([0 angle 0 2.5])
box

figure(4)
hold on
plot(xplot,shear,'-k')
xlabel('\theta/\pi','fontsize',15)
ylabel('H','fontsize',15)
set(gca,'fontsize',15)
axis([0 angle 2 4])
box

%---
% done
%---

return

%================================
function f = phase_vel (U0,tht,Lam,lam)

  tmp = 37.0-Lam/3.0-5.0*Lam*Lam/144.0;
  H = 31.5*(3.0-Lam/12.0)/tmp;
  S = (2.0+Lam/6.0)*tmp/315.0;
  U  = 2.0*U0*sin(tht);
  f = 2.0*(S-(2.0+H)*lam)/U;

return

%================================

function Lam = root(lam,Lam)

%---
% Solve for Lambda, given lambda
%---

tol = 0.000001;
eps = 0.000001;

 if(abs(lam)<0.000005)
   Lam = 0.00001;
   return
 end

%---
% Newton's method
%---

  Ic=0;
  for i=1:50
    fnc   = ltoL(lam,Lam);
    fnc1  = ltoL(lam,Lam+eps);
    fncp  = (fnc1-fnc)/eps;
    corr  = -fnc/fncp;
    Lam   = Lam+corr;
    Ic=Ic+1;
    if(abs(corr)<tol) 
      return; end
   end

   disp 'root: could not find a root in 50 iterations'
   pause

return

%================================

function f = ltoL(lam,Lam)

  f = Lam*( (37.0-Lam/3.0-5.0*Lam*Lam/144.0)/315.0 )^2 ...
   - lam;

return

%=================

function [lam,Lam] = Laminit(Lam)

 tol=0.000001;
 eps=0.000001;

%---
% Solve for the initial Lambda
% by Newton's method
%---

  Ic=0;
  for i=1:50
    fnc   = fLaminit(Lam);
    fnc1  = fLaminit(Lam+eps);
    fncp  = (fnc1-fnc)/eps;
    corr  = -fnc/fncp;
    Lam   = Lam+corr;
    Ic=Ic+1;
    if(abs(corr)<tol) 
      lam = Lam*( (37.0-Lam/3.0-5.0*Lam*Lam/144.0)/315.0 )^2;
      return; end
   end

   disp 'root: could not find a root in 50 iterations'
   pause

return

%================================

function f = fLaminit(Lam)

  lam = Lam*( (37.0-Lam/3.0-5.0*Lam*Lam/144.0)/315.0 )^2;
  tmp = 37.0-Lam/3.0-5.0*Lam*Lam/144.0;
  H     = 31.5*(3.0-Lam/12.0)/tmp;    % (10.6.21)
  S     = (2.0+Lam/6.0)*tmp/315.0;     % (10.6.21)
  f = S -(2+H)*lam;

return
