      program cvt_pm

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c-----------------------------------------------
c Transient flow in a rectangular cavity driven
c by the translation of the lid
c computed by a variant of Alexandre Chorin's
c projection method.
c
c Cavity inside the rectangle:
c ---------------------------
c
c   0 < x < Lx
c   0 < y < Ly
c
c The Poisson equation is solved on a non-staggered grid.
c
c SYMBOLS:
c -------
c
c Lx, Ly:  Cavity dimensions
c
c Nx, Ny:  Number of divisions in the x and y directions
c          yielding a uniform grid
c
c BCxu:
c
c    Boundary conditions for the x velocity
c    at the upper wall, to be used in
c    integrating the convection-diffusion
c    equation.
c
c BCxl:
c
c    Boundary conditions for the x velocity
c    at the lower wall, to be used in
c    integrating the convection-diffusion
c    equation.
c
c  (similar notation for the other two walls).
c
c atr,btr,ctr:
c
c    Entries of a tridiagonal matrix
c    used for integrating the covection-diffusion
c    equation
c
c    |a b ...   |
c    |c a b ... |
c    |...   c a |
c
c U, V: 
c
c    velocity components
c
c UU, VV:
c
c     Intermediate velocity
c
c T:
c
c     Projection function
c
c P:
c
c     Pressure
c
c Divus:
c
c     Divergence of u*
c
c Div:
c
c     Divergence of u
c
c
c OPTIONS
c -------
c
c Impl = 1 : Poisson equation is enforced at interior
c            AND boundary nodes
c        2 : Poisson equation is enforced at interior
c            nodes ALONE
c
c IGhia:  (applicable only for Impl = 2)
c          Flag for the location of the
c          grid layer where boundary conditions
c          for the PPE are required:
c
c          IGhia = 0:  The BC is applied at the boundary
c          coupled with biased extrapolation
c
c          IGhia = 1:  Ghia's method: the BC is applied
c          midway between the first and second layer
c
c IPBC:  Flag for the implementation of the
c        pressure boundary condition for the PPE:
c
c        0 : homogeneous Neumann  (wrong)
c        1 : homogeneous Neumann  (zeroth order)
c        2 : homogeneous Neumann  (first order)   *
c        4 : homogeneous Neumann  (second order)
c        5 : homogeneous Neumann  (third order)
c
c----------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision Lx,Ly

      Dimension  X(129,129), Y(129,129)
      Dimension  U(129,129), V(129,129)
      Dimension UU(129,129),VV(129,129)

      Dimension BCxb(129),BCxu(129)
      Dimension BCyl(129),BCyr(129)
      Dimension       Vlid(-1:131)

      Dimension T(0:130,0:130)
      Dimension P(0:130,0:130)
      Dimension S(0:130,0:130)

      Dimension Divus(129,129)
      Dimension Divu (129,129)

      Dimension    Blin(16666)

      Dimension atr(129),btr(129),ctr(129)
      Dimension Blu(129),Blv(129)
      Dimension Xsu(129),Xsv(129)

      Dimension pnbcl(129),pnbcr(129)  ! left and right poisson
                                     ! Neumann boundary conditions
      Dimension pnbcb(129),pnbct(129)  ! bottom and top poisson
                                     ! Neumann boundary conditions

      Dimension Compb(10000),Compa(10000)

c--------------
c common blocks
c--------------

      common/XXYY/X,Y
      common/UUVV/U,V
      common/DIVE/Divu

      common/pnbc/pnbcl,pnbcr,pnbcb,pnbct  ! Poisson Neumann BC
      common/piii/pi,pi2,pi4

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi
      pis = pi**2

      zero = 0.0D0

      Null  = 0
      None  = 1
      Ntwo  = 2
      Nfive = 5

c----------------
c Flow parameters
c----------------

 91   Continue
 
c     write (6,*) 
c     write (6,*) " Enter:"
c     write (6,*) 
c     write (6,*) " 1 to read parameters from file: cvt_pm.dat"
c     write (6,*) "   (recommended)"
c     write (6,*) " 2 to enter parameters from the keyboard"
c     write (6,*) " 0 to quit"
c     write (6,*) " ---------"
c     read  (5,*) Ienrd

      Ienrd = 1

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.2) then
c------------------------

      write (6,*)
      write (6,*) " Please enter the lid velocity"
      write (6,*) " -----------------------------"
      read  (5,*) Vlidamp

      write (6,*)
      write (6,*) " Choose the lid type:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1  for constant-impulsive"
      write (6,*) " 10 for constant-ramped"
      write (6,*) " 2  for sinusoidal-impulsive"
      write (6,*) " 20 for sinusoidal-ramped"
      write (6,*) " 3  for quadratic-impulsive"
      write (6,*) " 30 for quadratic-ramped"
      write (6,*) " --------------------------"
      read  (5,*) lid_type

      If(lid_type.eq.0) Go to 99

      If(lid_type.eq.10.or.lid_type.eq.20
     +                 .or.lid_type.eq.30
     +  ) then
       write (6,*)
       write (6,*) " Enter the ramp-up time constant"
       write (6,*) " -------------------------------"
       read  (5,*) tau
      End If

      write (6,*)
      write (6,*) " Enter the box dimensions Lx, Ly"
      write (6,*) " -------------------------------"
      read  (5,*) Lx,Ly

      write (6,*)
      write (6,*) " Enter the discretization levels Nx, Ny"
      write (6,*) " corresponding to the x and y dimensions" 
      write (6,*) " ---------------------------------------"
      read  (5,*) Nx,Ny

      write (6,*)
      write (6,*) " Enter the viscosity of the fluid"
      write (6,*) " --------------------------------"
      read  (5,*) visc

      write (6,*)
      write (6,*) " Enter the density of the fluid"
      write (6,*) " -------------------------------"
      read  (5,*) rho

      write (6,*)
      write (6,*) " Enter the time step Dt"
      write (6,*) " ----------------------"
      read  (5,*) Dt

      write (6,*)
      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " The maximum number of iterations"
      write (6,*) " for solving the Poisson equation"
      write (6,*) " --------------------------------"
      read  (5,*) Itermax

      write (6,*)
      write (6,*) " Enter the accuracy"
      write (6,*) " ------------------"
      read  (5,*) tol


      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " the maximum number of iterations"
      write (6,*) " for the projection function"
      write (6,*) " --------------------------------"
      read  (5,*) Nphi

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " the slip-velocity tolerance"
      write (6,*) " ---------------------------"
      read  (5,*) sliptol

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " the relaxation constant for the"
      write (6,*) " iterations on the slip velocity"
      write (6,*) " -------------------------------"
      read  (5,*) relax

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to initialize the velocity to zero"
      write (6,*) " 1 to read initial velocity"
      write (6,*) "   from file: cvt_pm.inp"
      write (6,*) " -----------------------"
      read  (5,*) Iread_vel

      write (6,*)
      write (6,*) " Compute the pressure?"
      write (6,*) 
      write (6,*) " 0 for NO, 1 for YES"
      write (6,*) " -------------------"
      read  (5,*) Icomp_P

      write (6,*)
      write (6,*) " How many time steps ?"
      write (6,*) " ---------------------"
      read  (5,*) Nstep

      write (6,*)
      write (6,*) " Please enter the final time?"
      write (6,*) " ----------------------------"
      read  (5,*) tend

      write (6,*)
      write (6,*) " Enter 1 to see intermediate velocities"
      write (6,*) "       0 to suppress them"
      write (6,*) " --------------------------------------"
      read  (5,*) IseeV

      write (6,*)
      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to see rate of expansion on the screen"
      write (6,*) " 0 to suppress it"
      write (6,*) " ----------------------------------------"
      read  (5,*) IseeE

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to see the projection function"
      write (6,*) " 0 to suppress it"
      write (6,*) " ---------------------------------"
      read  (5,*) IseeT

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to see the pressure"
      write (6,*) " 0 to suppress it"
      write (6,*) " ---------------------------------"
      read  (5,*) IseeP

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to see profiles of phi and P"
      write (6,*) " 0 to suppress it"
      write (6,*) " ---------------------------------"
      read  (5,*) IseeTP

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to see the lid vorticity"
      write (6,*) "       0 to suppress it"
      write (6,*) " ---------------------------------"
      read  (5,*) IseeO

c------------------------------
      Else If (Ienrd.eq.1) then
c------------------------------

      open (7,file="cvt_pm.dat")

        read (7,*) Vlidamp
        read (7,*) lid_type
        read (7,*) tau
        read (7,*) 
        read (7,*) Lx,Ly
        read (7,*) Nx,Ny
        read (7,*) 
        read (7,*) visc
        read (7,*) rho
        read (7,*) 
        read (7,*) Dt
        read (7,*) Delh
        read (7,*) 
        read (7,*) Impl
        read (7,*) 
        read (7,*) Itermax
        read (7,*) tol
        read (7,*) 
        read (7,*) Nphi
        read (7,*) sliptol
        read (7,*) relax
        read (7,*) 
        read (7,*) Iread_vel
        read (7,*)
        read (7,*) Icomp_P
        read (7,*) IPBC
        read (7,*) IGhia
        read (7,*) 
        read (7,*) Nstep
        read (7,*) tend
        read (7,*) 
        read (7,*) IseeJ
        read (7,*) IseeV
        read (7,*) IseeE
        read (7,*) IseeT
        read (7,*) IseeP
        read (7,*) IseePS
        read (7,*) IseeTP
        read (7,*) IseeO

      close (7)

c     If(Iread_vel.eq.1) then
c      write (6,*)
c      write (6,*) " Please re-enter the lid velocity"
c      write (6,*) " --------------------------------"
c      read  (5,*) Clidamp
c     End If

c---------
      else
c---------

        write (6,*) " Must enter 0 or 1; please try again"

        Go to 91

c-----------
      end if
c-----------

c---------------------
c END OF READING INPUT
c---------------------

c--------
c prepare
c--------

      rnu = visc/rho   ! kinematic viscosity

      Nxb = Nx-2
      Nxa = Nx-1
      Nx1 = Nx+1
      Nx2 = Nx+2

      Nyb = Ny-2
      Nya = Ny-1
      Ny1 = Ny+1
      Ny2 = Ny+2

c----------
c steps etc
c----------

      Dx = Lx/(Nx1-1.0D0)
      Dy = Ly/(Ny1-1.0D0)

      Dx2 = 2.0D0*Dx
      Dy2 = 2.0D0*Dy

      Dxs = Dx**2
      Dys = Dy**2

c------
c over-ride the time step !!!!!!
c------

      Dt = Delh**2 * Dx**2/rnu

      write (6,*) " Time step set at :",Dt
      pause

      Dtor = Dt/rho

c-----------------------------------
c Define the grid points
c Initialize the projection function
c-----------------------------------

      Do j=1,Ny1
       Do i=1,Nx1
         X(i,j) = (i-1.0D0)*Dx
         Y(i,j) = (j-1.0D0)*Dy
         T(i,J) = 0.0D0            ! projection function
       End Do
      End Do

c-----------------------------
c Initialize time and velocity
c-----------------------------

      write (6,*) " cvt_pm: initializing"

      time = 0.0D0        ! initialize time

      Do j=1,Ny1
       Do i=1,Nx1
         U(i,j) = 0.0D0
         V(i,j) = 0.0D0
       End Do
      End Do

c---------------------------
c lid velocity - impulsive
c
c Vlid will be held constant
c---------------------------

        If(lid_type.eq.1
     + .or.lid_type.eq.2
     + .or.lid_type.eq.3
     +    ) then

       Vamp = Vlidamp

       Do i=0,Nx+2

        if(lid_type.eq.1) then
        Vlid(i) = Vamp
        elseif(lid_type.eq.2) then
         arg = pi2*(i-1.0D0)*Dx/Lx
         Vlid(i) = 0.50D0*Uamp*(1.0-dcos(arg))
        elseif(lid_type.eq.3) then
         xh = (i-1.0D0)/(Nx1-1.0D0)
         Vlid(i) = 16.0D0 * Uamp* xh**2 * (1.0D0-xh)**2
        end if

        End Do

        Do i=1,Nx1
         U(i,Ny1) = Vlid(i)       ! lid
        End Do

       End If

c----------------------------
      if(Iread_vel.eq.1) then   ! read initial velocity
c----------------------------

      open (4,file="cvt_pm.inp")

c-----
c this file should be a copy of: cvt_pm.vel
c-----

      read (4,*)
      read (4,*)
      read (4,*)
      read (4,*)
      read (4,*)
      write (6,*) Nx1
      Do j=1,Ny1
        read (4,*) (U(i,j),i=1,Nx1)
      End Do
      read (4,*)
      read (4,*)
      read (4,*)
      read (4,*)
      read (4,*)
      write (6,*) Ny1
      Do j=1,Ny1
        read (4,*) (V(i,j),i=1,Nx1)
      End Do

      write (6,*)
      write (6,*) "Please enter the initial time"
      read  (5,*) time

      close (4)

c-----------
      End If
c-----------

c-------------------------------------
c no-slip velocity boundary conditions
c-------------------------------------

      Do i=1,Nx1
        BCxu(i) = Vlid(i)  ! upper wall
        BCxb(i) = 0.0D0    ! bottom wall
      End Do

      Do j=1,Ny1
        BCyl(j) = 0.0D0    ! left wall
        BCyr(j) = 0.0D0    ! right wall
      End Do

c-.-.-.-.-.-.-.
c TIME STEPPING
c-.-.-.-.-.-.-.

      write (6,*) " cvt_pm: begin time stepping"

      Kstep = 1   ! time step counter

      open (2,file="cvt_pm.comp")

 93   Continue   ! Return here for another time step

c----------------------
c lid velocity - ramped
c----------------------

        If(lid_type.eq.10
     + .or.lid_type.eq.20
     + .or.lid_type.eq.30
     +    ) then

c       arg = time*rnu/(tau*Lx**2)
        arg = time*Vlidamp/(tau*Lx)
        Uamp = Vlidamp*(1.0D0-dexp(-arg))

        Do i=0,Nx+2
         if(lid_type.eq.10) then
          Vlid(i) = Uamp
         elseif(lid_type.eq.20) then
          arg = pi2*(i-1.0D0)*Dx/Lx
          Vlid(i) = 0.50*Uamp*(1.0D0-dcos(arg))
         elseif(lid_type.eq.30) then
          xh = (i-1.0D0)/(Nx1-1.0D0)
          Vlid(i) = 16.0D0 * Uamp* xh**2 * (1.0D0-xh)**2
         end if
        End Do

        Do i=1,Nx1
         U(i,Ny1) = Vlid(i)       ! lid
        End Do

c----
c new no-slip velocity boundary conditions
c----

        Do i=1,Nx1
          BCxu(i) = Vlid(i)  ! upper wall
          BCxb(i) = 0.0D0    ! bottom wall
        End Do

        Do j=1,Ny1
          BCyl(j) = 0.0D0    ! left wall
          BCyr(j) = 0.0D0    ! right wall
        End Do

c----------
      End If
c----------

c-------------------------------------
c Initialize the intermediate velocity
c-------------------------------------

      Do j=1,Ny1
       Do i=1,Nx1
        UU(i,j) = U(i,j) 
        VV(i,j) = V(i,j)
       End Do
      End Do

c----------------------------
c Will perform inner iterations
c for the projection function
c to satisfy the no-slip
c boundary condition
c----------------------------

      write (6,*) " Iteration, slip velocity"

      Do 97 iter=1,Nphi

c-------------------------------------------------
c  Integrate Conv-Diff equation in the x direction
c  using the Crank-Nicolson method
c
c  Equation (12.6.23) of Pozrikidis (1997, p. 592)
c
c  Advance the velocity from u^n to u^*
c-------------------------------------------------

c---
c zero the entries of the tridiagonal matrix
c---

      Do i=1,Nxa
       atr(i) = 0.0D0
       btr(i) = 0.0D0
       ctr(i) = 0.0D0
      End Do

c---
c run over rows
c---

      RL = rnu*Dt/Dxs   

c----
      Do j=2,Ny             ! run over rows
c----
       Do i=2,Nx

        Ia = I-1
        I1 = I+1

        RC = U(i,j)*Dt/Dx     ! cell Reynolds number

        C1      =   RC + 2.0D0*RL
        C2      =        4.0D0*(1.0D0-RL)
        C3      =  -RC + 2.0D0*RL
        ctr(Ia) =  -RC - 2.0D0*RL
        atr(Ia) =        4.0D0*(1.0D0+RL)
        btr(Ia) =   RC - 2.0D0*RL

        Blu(Ia) = C1*U(Ia,J) + C2*U(I,J) + C3*U(I1,J)  ! right-hand side
        Blv(Ia) = C1*V(Ia,J) + C2*V(I,J) + C3*V(I1,J)  ! right-hand side

       End Do
c---

       Blv(1)   = Blv(1)   - ctr(1)  *BCyl(J)
       Blv(Nxa) = Blv(Nxa) - btr(Nxa)*BCyr(J)

       call thomas            ! x component
     +
     +  (Nxa,atr,btr,ctr,Blu
     +  ,Xsu
     +  ) 

       call thomas            ! y component
     +
     +  (Nxa,atr,btr,ctr,Blv
     +  ,Xsv
     +  )

       Do k=1,Nxa
        K1 = K+1
        UU(k1,j) = Xsu(k)
        VV(k1,j) = Xsv(k)
       End Do

c---
      End Do       ! End of running over rows
c---

      Do i=1,Nya        ! reset the entries of the tridiagonal matrix
       atr(i) = 0.0D0
       btr(i) = 0.0D0
       ctr(i) = 0.0D0
      End Do

c-----------------
c printing session
c-----------------

      If(IseeV.eq.1) then

        write (6,*)
        write (6,*) " Velocities after x integration"
        write (6,*) " ------------------------------"
        write (6,*)

        Do J=1,Ny1
          write (6,100) (UU(I,J),I=1,Nx1)
        End Do

        write (6,*)
        Do J=1,Ny1
          write (6,100) (VV(I,J),I=1,Nx1)
        End Do

      End If

c-------------------------------------------------
c  Integrate Conv-Diff equation in the y direction
c
c  Advance the velocity from u^* to u^**
c-------------------------------------------------

c---
c run over columns
c---

      RL = rnu*Dt/Dys

c---
      Do i=2,Nx      ! run over columns
c---
       Do J=2,Ny     ! from bottom to top

        Ja = J-1
        J1 = J+1

        RC = V(I,J)*Dt/Dy       ! cell Reynolds number

        C1      =  RC +2.0D0*RL
        C2      =      4.0D0*(1.0D0-RL)
        C3      = -RC +2.0D0*RL
        ctr(Ja) = -RC -2.0D0*RL
        atr(Ja) =      4.0D0*(1.0D0+RL)
        btr(Ja) =  RC -2.0*RL

        Blu(Ja) = C1*UU(I,Ja) + C2*UU(I,J) + C3*UU(I,J1) ! right-hand side
        Blv(Ja) = C1*VV(I,Ja) + C2*VV(I,J) + C3*VV(I,J1) ! right-hand side

       End Do
c---

       Blu(1)   = Blu(1)   - ctr(1)  *BCxb(i)    ! boundary conditions
       Blu(Nya) = Blu(Nya) - btr(Nya)*BCxu(i)    ! boundary conditions

       call thomas      ! x component
     +
     +   (Nya,atr,btr,ctr
     +   ,Blu
     +   ,Xsu
     +   )

       call thomas      ! y component
     +
     +   (Nya,atr,btr,ctr
     +   ,Blv
     +   ,Xsv
     +   )

       Do K=1,Nya
         K1 = K+1
         UU(I,K1) = Xsu(K)
         VV(I,K1) = Xsv(K)
       End Do

c---
      End Do              ! End of run over columns
c---

c-----------------
c printing session
c-----------------

      If(IseeV.eq.1) then

        write (6,*)
        write (6,*) " Inner iteration: ",Iter
        write (6,*)
        write (6,*) " Velocity after y and x integration"
        write (6,*) " ----------------------------------"
        write (6,*)

        Do J=1,Ny1
          write (6,100) (UU(I,J),I=1,Nx1)
        End Do

        write (6,*)
        Do J=1,Ny1
          write (6,100) (VV(I,J),I=1,Nx1)
        End Do

      End If

c-------------------------------------
c Compute intermediate compressibility
c by centered differences
c
c Divus = Div u*
c-------------------------------------

! initialize

      Do j=1,Ny1
       Do i=1,Nx1
        Divus(i,j) = 0.0D0
       End Do
      End Do

! interior nodes

      Do i=2,Nx
       ia = i-1
       i1 = i+1
       Do j=2,Ny
        ja = j-1
        j1 = j+1
        DuDx = (UU(i1,j) - UU(ia,j))/Dx2 
        DvDy = (VV(i,j1) - VV(i,ja))/Dy2
        Divus(i,j) = DuDx+DvDy
       End Do
      End Do

! left wall

      Do j=1,Ny1
       DuDx = (-3.0*UU(1,j)+4.0D0*UU(2,j)-UU(3,j))/Dx2
       DvDy = 0.0D0
       Divus(1,j) = DuDx+DvDy
      End Do

      save11l = Divus(1,j)
      save12l = Divus(1,Ny1)

! bottom wall

      Do i=1,Nx1
       DuDx = 0.0D0
       DvDy = (-3.0D0*VV(i,1)+4.0D0*VV(i,2)-VV(i,3))/Dy2
       Divus(i,1) = DuDx+DvDy
      End Do

      save11b = Divus(1,1)
      save21b = Divus(Nx1,1)

! right wall

      Do j=1,Ny1
       DuDx = (3.0D0*UU(Nx1,j)-4.0D0*UU(Nx,j)+UU(Nxa,j))/Dx2
       DvDy = 0.0D0
       Divus(Nx1,j) = DuDx+DvDy
      End Do

      save21r = Divus(Nx1,1)
      save22r = Divus(Nx1,Ny1)

! top wall

      Do i=1,Nx1
       DuDx = (Vlid(i+1)-Vlid(i-1))/Dx2 
       DvDy = (3.0D0*VV(i,Ny1)-4.0D0*VV(i,Ny)+VV(i,Nya))/Dy2
       Divus(i,Ny1) = DuDx+DvDy
      End Do

      save12t = Divus(1,  Ny1)
      save22t = Divus(Nx1,Ny1)

! corners by averaging

      Divus(1,    1) = 0.50D0*(save11l+save11b)
      Divus(1,  Ny1) = 0.50D0*(save12l+save12t)
      Divus(Nx1,  1) = 0.50D0*(save21b+save21r)
      Divus(Nx1,Ny1) = 0.50D0*(save22r+save22t)

c------------------------
c display compressibility
c------------------------

      If(IseeE.eq.1) then

       write (6,*)
       write (6,*) " Div(u*) before projection "
       write (6,*) " ------------------------ "
       write (6,*)

c      fc = rho/Dt
       fc = 1.0D0

       Do j=1,Ny1
         write (6,100) (fc*Divus(i,j),i=1,Nx1)
       End Do

      End If

c----------------------------------
c      open (9,file="tmp.dat")
c      Do J=1,Ny1
c       write (9,*) J
c       Do I=1,Nx1
c        write (9,111) I,J,Divus(I,J)
c       End Do
c      End Do
c      close (9)
c----------------------------------

c---
c Compute the areal integral of Divus(u)
c
c (optional)
c---

       Compr = 0.0D0

       Do j=1,Ny
        Do i=1,Nx
         Compr = Compr+Divus(i,j)**2
     +                +Divus(i,j+1)**2
     +                +Divus(i+1,j)**2
     +                +Divus(i+1,j+1)**2
        End Do
       End Do

       Compb(Kstep) = 0.25D0*Dx*Dy*Compr

c---------------------------------------
c Homogeneous Poisson-Neumann boundary
c conditions for the projection function
c---------------------------------------

      Do j=1,Ny1
        pnbcl(j) = 0.0D0
        pnbcr(j) = 0.0D0
      End Do

      Do i=1,Nx1
        pnbcb(i) = 0.0D0
        pnbct(i) = 0.0D0
      End Do

c----------------------------------
c Solve for the projection function
c by Gauss-Siedel (GS) iterations
c----------------------------------

c----------------------
      If(Impl.eq.1) then
c----------------------

c-----
c RHS of Poisson's equation for the
c projection function
c-----


      Ic = 0

      Do j=1,Ny1
       Do i=1,Nx1
        Ic = Ic+1
        Blin(Ic) = Divus(i,j)/Dtor
        Blin(Ic) = Dxs*Blin(Ic)
       End Do
      End Do

      call pois_gs1
     +
     +  (Nx,Ny
     +  ,Dx,Dy
     +  ,Blin
     +  ,Itermax     ! maximum number of GS iterations
     +  ,tol         ! GS tolerance
     +  ,T           ! solution matrix
     +  )

c----------------------------
      Else If(Impl.eq.2) then
c----------------------------

      Ic = 0

      Do j=2,Ny
       Do i=2,Nx
        Ic = Ic+1
        Blin(Ic) = Divus(i,j)/Dtor
        Blin(Ic) = Dxs*Blin(Ic)
       End Do
      End Do

      Igs2 = 0   ! boundary condition is applied at 
                 ! boundary nodes 

      call pois_gs2
     +
     +  (Nx,Ny
     +  ,Dx,Dy
     +  ,Blin
     +  ,Igs2
     +  ,Itermax
     +  ,tol
     +  ,T
     +  )

      End If

c--------------------------------
c Display the projection function 
c--------------------------------

      If(IseeT.eq.1) then

       write (6,*)
       write (6,*) " cvt_pm: projection function at time", time
       write (6,*)

       Toff = T(Nx/2+1, Ny/2+1)

       Tmax = 0.0D0

       Do j=1,Ny1
        Do i=1,Nx1
         Rmax = abs(T(i,j)-Toff)
         If(Tmaxt.gt.Tmax) Tmax = Rmax
        End Do
       End Do
 
       Do j=1,Ny1
        write (6,110) (T(i,j)-Toff,i=1,Nx1)
c       write (6,110) ((T(i,j)-Toff)/Tmax,i=1,Nx1)
       End Do

       Tmax = 1.0D0
 
      End If

c----------------------------------
c Project the velocity at all nodes
c except at the corner nodes
c----------------------------------

      Do i=2,Nx

       ib = i-2
       ia = i-1
       i1 = i+1
       i2 = i+2

       Do J=2,Ny

        jb = j-2
        ja = j-1
        j1 = j+1
        j2 = j+2

        DTDx = (T(I1,J)-T(Ia,J))/Dx2
        DTDy = (T(I,J1)-T(I,Ja))/Dy2

        UU(i,j) = UU(i,j) - Dtor*DTDx
        VV(i,j) = VV(i,j) - Dtor*DTDy

       End Do
      End Do

c--------------
c Lower boundary
c
c Use forward differences with
c special treatment of the near-corner nodes
c--------------

      Do i=2,Nx

        DTDy = 0.0D0

        If(i.eq.2) then
	    DTDx = (-3.0D0*T(2,1)+4.0D0*T(3,1)-T(4,1))/Dx2
        Else If(i.eq.Nx) then
            DTDx =-(-3.0D0*T(Nx,1)+4.0D0*T(Nxa,1)-T(Nxb,1))/Dx2
	Else
            DTDx = (T(I+1,1)-T(I-1,1))/Dx2
	End If

        UU(i,1) = BCxb(i) - Dtor*DTDx
        VV(i,1) =         - Dtor*DTDy

      End Do

c--------------
c Upper boundary
c
c Use backward differences
c special treatment for the near-corner nodes
c--------------

      Do i=2,Nx

        DTDy = 0.0D0

        If(i.eq.2) then
	    DTDx = (-3.0D0*T(2,Ny1)+4.0D0*T(3,Ny1)-T(4,Ny1))/Dx2
        Else If(i.eq.Nx) then
            DTDx =-(-3.0D0*T(Nx,Ny1)+4.0D0*T(Nxa,Ny1)-T(Nxb,Ny1))/Dx2
	Else
            DTDx = (T(I+1,Ny1)-T(I-1,Ny1))/Dx2
	End If

        UU(i,Ny1) = BCxu(i) - Dtor*DTDx
        VV(i,Ny1) =         - Dtor*DTDy

      End Do

c-------------
c Left boundary
c
c Use forward differences
c special treatment for the near-corner nodes
c-------------

      Do j=2,Ny

        DTDx = 0.0

        If(j.eq.2) then
	    DTDy = (-3.0*T(1,2)+4.0*T(1,3)-T(1,4))/Dy2
        Else If(j.eq.Ny) then
            DTDy =-(-3.0*T(1,Ny)+4.0*T(1,Nya)-T(1,Nyb))/Dy2
	Else
            DTDy = (T(1,j+1)-T(1,j-1))/Dy2
	End If

        UU(1,j) =         - Dtor*DTDx
        VV(1,j) = BCyl(j) - Dtor*DTDy

      End Do

c--------------
c Right boundary
c
c Use backward differences
c special treatment for the near-corner nodes
c--------------

      Do j=2,Ny

        DTDx = 0.0

        If(j.eq.2) then
	    DTDy = (-3.0*T(Nx1,2)+4.0*T(Nx1,3)-T(Nx1,4))/Dy2
        Else If(j.eq.Ny) then
            DTDy =-(-3.0*T(Nx1,Ny)+4.0*T(Nx1,Nya)-T(Nx1,Nyb))/Dy2
	Else
            DTDy = (T(Nx1,J+1)-T(Nx1,J-1))/Dy2
	End If

        UU(Nx1,j) =         - Dtor*DTDx
        VV(Nx1,j) = BCyr(j) - Dtor*DTDy

      End Do

c----------------------------------
c As an alternative, compute the velocity
c at the near-corner points
c by extrapolation
c
c This is a rough approach
c----------------------------------
 
c     UU(2  ,1  ) = 2.0*UU(3  ,1  ) - UU(4  ,1  )
c     UU(Nx ,1  ) = 2.0*UU(Nxa,1  ) - UU(Nxb,1  )
c     UU(2  ,Ny1) = 2.0*UU(3  ,Ny1) - UU( 4 ,Ny1)
c     UU(Nx ,Ny1) = 2.0*UU(Nxa,Ny1) - UU(Nxb,Ny1)
c     VV(1  ,2  ) = 2.0*VV(1  ,3  ) - VV(1  ,4  )
c     VV(1  ,Ny ) = 2.0*VV(1  ,Nya) - VV(1  ,Nyb)
c     VV(Nx1,2  ) = 2.0*VV(Nx1,3  ) - VV(Nx1,4  )
c     VV(Nx1,Ny ) = 2.0*VV(Nx1,Nya) - VV(Nx1,Nyb)

c-------------------------------------- 
c Compute the wall slip velocity
c
c and modify the boundary conditions
c for the intermediate (star) velocities
c-------------------------------------- 

      slipmax = 0.0D0

      Do I=1,Nx1
        corr = UU(i,Ny1)-Vlid(i)
        If(abs(corr).gt.slipmax) slipmax = corr
        BCxu(i) = BCxu(i) - relax * corr
        corr = UU(i,1)
        If(abs(corr).gt.slipmax) slipmax = corr
        BCxb(I) = BCxb(I) - relax * corr
      End Do

      Do j=1,Ny1
        corr = VV(1,j)
        If(abs(corr).gt.slipmax) slipmax = corr
        BCyl(J) = BCyl(J) - relax * corr
        corr = VV(Nx1,j)
        If(abs(corr).gt.slipmax) slipmax = corr
        BCyr(J) = BCyr(J) - relax * corr
      End Do

c------
c Check for stopping the inner iterations
c------

      write (6,300) iter,slipmax

      If(slipmax.lt.sliptol) Go to 98

c-------------
 97   Continue         ! end of loop for inner iterations
c-------------

      write (6,*) " cvt_pm: inner iterations for the projection"
      write (6,*) "          function did not converge"
      write (6,*) 
      write (6,*) "          Sorry ! "

      stop
c---------------------------
c End of inner iterations
c on the projection function
c---------------------------

  98  Continue

c-----------------------------------
c Update velocity to the final value
c-----------------------------------

      Do j=1,Ny1
       Do i=1,Nx1
        U(i,j) = UU(i,j)
        V(i,j) = VV(i,j)
       End Do
      End Do

c---
c velocity printing session
c---

      If(IseeV.eq.1) then

       write (6,*)
       write (6,*) "x-velocity after projection"
       write (6,*) "---------------------------"
       write (6,*)

       Do J=1,Ny1
        write (6,100) (U(I,J),I=1,Nx1)
       End Do

       write (6,*)
       write (6,*) "y-velocity after projection"
       write (6,*) "---------------------------"
       write (6,*)

       Do J=1,Ny1
         write (6,100) (V(I,J),I=1,Nx1)
       End Do

      End If

c--------------------------------
c Compute Div(u) after projection
c--------------------------------

      Do j=2,Ny
       Do i=2,Nx
        DuDx = (U(I+1,J)-U(I-1,J))/Dx2 
        DvDy = (V(I,J+1)-V(I,J-1))/Dy2
        Divu(i,j) = DUDx+DVDy
       End Do
      End Do

! left wall

      Do j=2,Ny
       DuDx = (-3.0D0*U(1,j)+4.0D0*U(2,j)-U(3,j))/Dx2
       DvDy = 0.0D0
       Divu(1,j) = DuDx + DvDy
      End Do

      save11l = Divu(1,j)
      save12l = Divu(1,Ny1)

! right wall

      Do j=2,Ny
       DUDx = (3.0D0*U(Nx1,j)-4.0*U(Nx,j)+U(Nxa,j))/Dx2
       DVDy = 0.0D0
       Divu(Nx1,j) = DuDx+DvDy
      End Do

      save21r = Divu(Nx1,  1)
      save22r = Divu(Nx1,Ny1)

! bottom wall

      Do i=2,Nx
       DuDx = 0.0D0
       DvDy = (-3.0D0*V(i,1)+4.0*V(i,2)-V(i,3))/Dy2
       Divu(i,1) = DuDx+DvDy
      End Do

      save11b = Divu(1,  1)
      save21b = Divu(Nx1,1)

! top wall

      Do i=2,Nx
       DuDx = (U(i+1,Ny1)-U(i-1,Ny1))/Dx2
       DvDy = (3.0D0*V(i,Ny1)-4.0D0*V(i,Ny)+V(i,Nya))/Dy2
       Divu(i,Ny1) = DuDx+DvDy
      End Do

      save12t = Divu(1,  Ny1)
      save22t = Divu(Nx1,Ny1)

! corners by averaging

      Divu(1,    1) = 0.50D0*(save11l+save11b)
      Divu(1,  Ny1) = 0.50D0*(save12l+save12t)
      Divu(Nx1,  1) = 0.50D0*(save21b+save21r)
      Divu(Nx1,Ny1) = 0.50D0*(save22r+save22t)

c--------
c display
c--------

      If(IseeE.eq.1) then

       write (6,*)
       write (6,*) " cvt_pm: Div(u) after projection:"
       write (6,*)

c      fc = rho/Dt
       fc = 1.0D0

       Do j=1,Ny1
         write (6,100) (fc*Divu(i,j),i=1,Nx1)
       End Do

      End If

c---
c Compute the areal integral of Div(u)
c---

       Compr = 0.0D0

       Do j=1,Ny
        Do i=1,Nx
         Compr = Compr+Divu(i,j)**2
     +                +Divu(i,j+1)**2
     +                +Divu(i+1,j)**2
     +                +Divu(i+1,j+1)**2
        End Do
       End Do

       Compa(Kstep) = 0.25*Dx*Dy*Compr

c-----
c Record compressibility
c-----

      write (6,888)            Compb(Kstep),Compa(Kstep)
      write (2,134) Kstep,time,Compb(Kstep),Compa(Kstep)

c-----------
c Reset time
c-----------

      time = time + Dt

c     write (6,881) Kstep,time

c------------------------------
c Compute offset and max of the
c projection function
c------------------------------

      Toff = T(Nx/2+1,Ny/2+1)

      Tmax = 0.0D0

      Do J=1,Ny1
        Do I=1,Nx1
         Rmax = abs(T(I,J)-Toff)
         If(Rmax.gt.Tmax) Tmax = Rmax
        End Do
      End Do

      Tmax = 1.0D0

c--------------------------------
c Display the projection function 
c--------------------------------

      If(IseeT.eq.1) then

       write (6,*)
       write (6,*) " cvt_pm: projection function at time: ",time
       write (6,*)
 
       Do j=1,Ny1
         write (6,110) (T(i,j)-Toff,i=1,Nx1)
       End Do
 
      End If

c-----------------
c Printing session
c-----------------

c     write (6,*)
c     write (6,889) time
c     write (6,*)

c     write (6,*)
c     write (6,*) " Solution for phi  "
c     write (6,*) " ----------------- "
c     write (6,*)

c     Do J=1,Ny1
c       write (6,100) (T(I,J),I=1,Nx1)
c     End Do

c     write (6,*)
c     write (6,*) " x-velocity"
c     write (6,*) " ----------"
c     write (6,*)

c     Do J=1,Ny1
c       write (6,100) (U(I,J),I=1,Nx1)
c     End Do

c     write (6,*)
c     write (6,*) " y-velocity"
c     write (6,*) " ----------"
c     write (6,*)

c     Do J=1,Ny1
c       write (6,100) (V(I,J),I=1,Nx1)
c     End Do

c--------------------------
c PRESSURE POISSON EQUATION (PPE)
c
c (optional)
c--------------------------

      If(Icomp_P.eq.0) Go to 777

c-------------
       if(Impl.eq.1) then
c-------------

       call cvt_pm_press1
     +
     +  (Lx,Ly
     +  ,Nx,Ny
     +  ,Vlid
     +  ,visc,rho
     +  ,Uamp
     +  ,IPBC
     +  ,IseePS
     +  ,Itermax
     +  ,tol
     +  ,P
     +  )

c-------------
       elseif(Impl.eq.2) then
c-------------

       Igs2 = IGhia

       call cvt_pm_press2
     +
     +  (Lx,Ly
     +  ,Nx,Ny
     +  ,Vlid
     +  ,visc,rho
     +  ,Uamp
     +  ,Igs2
     +  ,IPBC
     +  ,IseePS
     +  ,Itermax
     +  ,tol
     +  ,P
     +  )

c-------------
       End If
c-------------

c-----
c Compute the corrected pressure
c-----

      Do i=1,Nx1
       Do j=1,Ny1
         S(i,j) = T(i,j) - 0.50D0*visc*Divus(i,j)
       End Do
      End Do

      Soff = P(Nx/2+1,Ny/2+1)  ! pressure offset
      Smax = 1.0D0

c-----
c Compute the pressure offset
c and maximum
c-----

      Poff = P(Nx/2+1,Ny/2+1)  ! pressure offset

      Pmax = 0.0D0

      Do j=1,Ny1
        Do i=1,Nx1
         Rmax = abs(P(i,j)-Poff)
         If(Rmax.gt.Pmax) Pmax = Rmax
        End Do
      End Do

c---------------------
c display the pressure
c---------------------

      If(IseeP.eq.1) then

       write (6,*)
       write (6,*) " cvt_pm: pressure at time: ",time
       write (6,*)

       Do j=1,Ny1
cc       write (6,110) ((P(i,j)-Poff)/Pmax,i=1,Nx1)
         write (6,110) (P(i,j)-Poff,i=1,Nx1)
       End Do

      End If

c--------------------------
c phi and pressure profiles
c--------------------------

      Tmax = 1.0D0
      Pmax = 1.0D0

      If(IseeTP.eq.1) then

c------

c      Go to 333

       write (6,*)
       write (6,*) " cvt_pm: horiz prof phi, p: ",time
       write (6,*)

       write (6,*) Nx1,time

       Do i=1,Nx1
        j = Ny1
        pr01 = (T(i,j)-Toff)/Tmax
        pr02 = (P(i,j)-Poff)/Pmax
        pr03 = (S(i,j)-Soff)/Smax
        j = Ny
        pr11 = (T(i,j)-Toff)/Tmax
        pr12 = (P(i,j)-Poff)/Pmax
        pr13 = (S(i,j)-Soff)/Smax
        j = Ny/2+1
        pr21 = (T(i,j)-Toff)/Tmax
        pr22 = (P(i,j)-Poff)/Pmax
        pr23 = (S(i,j)-Soff)/Smax
        write (6,112) i,X(i,j)
     +                 ,pr01,pr02,pr03
     +                 ,pr11,pr12,pr13
     +                 ,pr21,pr22,pr23
       End Do

  333  Continue
c------

       write (6,*)
       write (6,*) " cvt_pm: vert profile phi, p: ",time
       write (6,*)

       write (6,*) Ny1,time

       Do j=1,Ny1
        i = Nx/2+1
        pr01 = (T(i,j)-Toff)/Tmax
        pr02 = (P(i,j)-Poff)/Pmax
        pr03 = (S(i,j)-Soff)/Pmax
        write (6,112) j,Y(i,j)
     +                 ,pr01,pr02,pr03
       End Do

       write (6,*)

       End If

c-----------------------
c End of Pressure module
c-----------------------

  777 Continue

c---------------------
c Vorticity at the top 
c---------------------

      If(IseeO.eq.1) then

      write (6,*)
      write (6,*)  " Vorticity at the top"
      write (6,*)

      write (4,106) Nx-1

      Do i=2,Nx
       vort = - (3.0D0*U(i,Ny1)-4.0D0*U(i,Ny)+U(i,Nya))/Dy2
       xh = (i-1.0D0)/(Nx1-1.0D0)
       Ocorner = 0.0D0
       If(xh.ne.0) then
         Ocorner = - pi4/(pis-4.0D0) * UUlid/xh
       End If
       write (6,106) i,xh,vort,Ocorner
      End Do

      write (4,106) Null

      End If

c------------------------
c return for another step
c------------------------

      If(Kstep.lt.Nstep.and.time.lt.tend)  then
        Kstep = Kstep + 1
        Go to 93
      End If

c-.-.-.-.-.-.-.-.-.-.-.
c END OF THE SIMULATION
c-.-.-.-.-.-.-.-.-.-.-.

  99  Continue

c------------------------------------
c Record output for plotting velocity
c vector field (vvf)
c------------------------------------

      open (4,file="cvt_pm.vvf")

c---
c print cavity walls
c---

      write (4,104) Nfive
      write (4,104) None,None,zero,zero
      write (4,104) None,None,Lx  ,zero
      write (4,104) None,None,Lx  ,Ly
      write (4,104) None,None,zero,Ly
      write (4,104) None,None,zero,zero

      Umax = 0.0D0

      Do j=2,Ny1
       Do i=2,Nx1
        Umag = U(i,j)**2+V(i,j)**2
        If(Umag.gt.Umax) Umax = Umag
       End Do
      End Do

      Umax = sqrt(Umax)

      Do j=1,Ny1
       Do i=1,Nx1
        Xv = X(i,j) + 0.1*U(i,j)/Umax
        Yv = Y(i,j) + 0.1*V(i,j)/Umax
        write (4,104) Ntwo
        write (4,104) j,i,X(i,j),Y(i,j)
        write (4,104) j,i,Xv,Yv
       End Do
      End Do

      write (4,*)
      write (4,400 ) UUlid
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c--------------------------------
c  record the final top vorticity
c--------------------------------

      open (4,file="cvt_pm.vort")

      write (4,106) Nx-1

      Do i=2,Nx
       vort = - (3.0D0*U(i,Ny1)-4.0D0*U(i,Ny)+U(i,Nya))/Dy2
       xh = (i-1.0D0)/(Nx1-1.0D0)
       Ocorner = 0.0D0
       if(xh.ne.0) then
         Ocorner = - pi4/(pis-4.0D0) * UUlid/xh
       end if
       write (4,106) i,xh,vort,Ocorner
      End Do

      write (4,106) Null
      write (4,*)
      write (4,*) "time=",time

      write (4,400 ) UUlid
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c----------------------------------
c  record final projection function
c----------------------------------

      open (4,file="cvt_pm.prj")

      write (4,*) Nx,Ny
      write (4,*) Dx,Dy

      Do j=1,Ny1
c       Do i=1,Nx1
c        write (4,104) j,i,X(i,j),Y(i,j),T(i,j)
c       End Do
        write (4,100) (T(i,j),i=1,Nx1)
      End Do

      write (4,*)
      write (4,*) "time=",time

      write (4,*)
      write (4,400 ) Vlidamp
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c------------------------------------------------
c  record the final corrected projection function
c------------------------------------------------

      open (4,file="cvt_pm.prjc")

      write (4,*) Nx,Ny
      write (4,*) Dx,Dy

      Do j=1,Ny1
c       Do i=1,Nx1
c        write (4,104) j,i,X(i,j),Y(i,j),T(i,j)
c       End Do
        write (4,100) (S(i,j),i=1,Nx1)
      End Do

      write (4,*)
      write (4,*) "time=",time

      write (4,*)
      write (4,400 ) Vlidamp
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)


c---------------------------
c  record the final pressure
c---------------------------

      open (4,file="cvt_pm.press")

      write (4,*) Nx,Ny
      write (4,*) Dx,Dy

      Do j=1,Ny1
        write (4,100) (P(i,j),i=1,Nx1)
      End Do

      write (4,*)
      write (4,*) "time=",time

      write (4,*)
      write (4,400 ) Vlidamp
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c----------------------
c  record final div(u*)
c----------------------

      open (4,file="cvt_pm.divus")

      write (4,*) Nx,Ny
      write (4,*) Dx,Dy

      Do j=1,Ny1
        write (4,100) (Divus(i,j),i=1,Nx1)
      End Do

      write (4,*)
      write (4,*) "time=",time
      write (4,*)
      write (4,*) "Div(u*) on the lid:"
      write (4,*)
      write (4,*) Nx1,Nx,Ny,Dt,time

      Do i=1,Nx1
        write (4,109) i,X(i,Ny1),Divus(i,Ny1)
      End Do
      write (4,*) Null

      write (4,*)
      write (4,400 ) Vlidamp
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c-----------------------
c  record final velocity
c-----------------------

      open (4,file="cvt_pm.vel")

      write (4,*)
      write (4,*)"x-velocity"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy

      Do j=1,Ny1
         write (4,100) (U(i,j),i=1,Nx1)
      End Do

      write (4,*)
      write (4,*)"y-velocity"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy
      Do j=1,Ny1
        write (4,100) (V(i,j),i=1,Nx1)
      End Do

      write (4,*)
      write (4,*) "time=",time

      write (4,*)
      write (4,400 ) Vlidamp
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c--------------------------
c Record phi and p profiles
c--------------------------

      open (4,file="cvt_pm.prp")

      write (4,*) Nx1,time," j=",Ny/2+1

      Do i=1,Nx1
        j = Ny/2+1
        pr01 = (T(i,j)-Toff)/Tmax
        pr02 = (P(i,j)-Poff)/Pmax
        pr03 = (S(i,j)-Soff)/Smax
        write (4,112) i,X(i,j)
     +                 ,pr01,pr02,pr03
      End Do

      write (4,*) Null,Null,Null

      write (4,*) Ny1,time," i=",Nx/2+1

       Do j=1,Ny1
        i = Nx/2+1
        pr01 = (T(i,j)-Toff)/Tmax
        pr02 = (P(i,j)-Poff)/Pmax
        pr03 = (S(i,j)-Soff)/Smax
        write (4,112) j,Y(i,j)
     +                 ,pr01,pr02,pr03
       End Do

      write (4,*) Null,Null,Null

      close (4)

c-----------
c Record all
c-----------

      open (4,file="cvt_pm.all")

      write (4,*)
      write (4,*)" cvt_pm: Projection function:"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy
      Do j=1,Ny1
        write (4,100) (T(i,j),i=1,Nx1)
      End Do
c---
      write (4,*)
      write (4,*)"Pressure"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy
      Do j=1,Ny1
        write (4,100) (P(i,j),i=1,Nx1)
      End Do
c---
      write (4,*)
      write (4,*)"x-velocity"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy
      Do j=1,Ny1
        write (4,100) (U(i,j),i=1,Nx1)
      End Do
c---
      write (4,*)
      write (4,*)"y-velocity"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy
      Do j=1,Ny1
        write (4,100) (V(i,j),i=1,Nx1)
      End Do
c---
      write (4,*)
      write (4,*)"Div(u*)"
      write (4,*)
      write (4,*) Nx,Ny
      write (4,*) Dx,Dy
      Do j=1,Ny1
        write (4,100) (Divus(i,j),i=1,Nx1)
      End Do
c---
      write (4,*)
      write (4,*)"horizontal phi and pressure profiles:"
      write (4,*)
      write (4,*) Nx1,time

      Do i=1,Nx1
        j = Ny/2+1
        pr01 = (T(i,j)-Toff)/Tmax
        pr02 = (P(i,j)-Poff)/Pmax
        pr03 = (S(i,j)-Soff)/Smax
        write (4,112) i,X(i,j)
     +                 ,pr01,pr02,pr03
      End Do

      write (4,*)
      write (4,*)"vertical phi and pressure profiles:"
      write (4,*) Ny1,time

      Do j=1,Ny1
       i = Nx/2+1
       pr01 = (T(i,j)-Toff)/Tmax
       pr02 = (P(i,j)-Poff)/Pmax
       pr03 = (S(i,j)-Soff)/Smax
       write (4,112) j,X(i,j)
     +                ,pr01,pr02,pr03
      End Do

      write (4,*) Null,Null,Null

      write (4,*)
      write (4,*)"vertical velocity profile:"
      write (4,*) Ny1,time

      Do j=1,Ny1
       i = Nx/2+1
       pr01 = U(i,j)
       pr02 = V(i,j)
       pr03 = 0.0D0
       write (4,112) j,X(i,j)
     +                ,pr01,pr02,pr03
      End Do

      write (4,*) Null,Null,Null
c---
      write (4,*)
      write (4,*)"Compressibility before and after:"
      write (4,*)
      write (4,*) Kstep
      Do i=1,Kstep
       write (4,134) i,(i-1.0)*Dt,Compb(i),Compa(i)
      End Do

c----

      write (4,*)
      write (4,*)
      write (4,*)
      write (4,*) "time=",time
      write (4,*)
      write (4,*)
      write (4,400 ) Vlidamp
      write (4,4001) lid_type
      write (4,4002) tau
      write (4,401) Lx,Ly
      write (4,402) Nx,Ny
      write (4,403) visc
      write (4,404) rho
      write (4,405) Dt
      write (4,409) Itermax
      write (4,410) tol
      write (4,411) Nphi
      write (4,412) sliptol
      write (4,413) tend
      write (4,414) IPBC
      write (4,415) IGhia
      write (4,420) Impl
      write (4,421) Delh

      close (4)

c-----
c Done
c-----

      close (2)

 100  Format (98(1x,f8.4))
 101  Format (2(1x,i3),2(1x,f5.3))
 104  Format (2(1x,i2),20(1x,f10.5))
 106  Format (1x,i4,20(1x,f15.5))

 109  Format (1x,i4,1x,f10.5,1x,f15.10)
 110  Format (30(1x,f8.3))
 111  Format (2(1x,I3),1x,f15.8)
 112  Format (1x,I3,1x,f9.6,10(1x,3(1x,f12.8)))
 134  Format (1x,I4,f15.10,2(1x,f15.10))

 881  Format (" step: ",I6,"  time: ",f12.6)
 888  Format (" Int {Div(u)}^2 dS before and after: ",f15.10,1x,f15.10)
 889  Format (" Velocity at time : ",f10.7)

 300  Format (1x,I2,1x,f15.10)

 400  Format (1x," Vlidamp     ",F10.5)
 4001 Format (1x," lid_type    ",I3)
 4002 Format (1x," tau         ",f10.5)
 401  Format (1x," Lx,Ly       ",F10.5,1x,F10.5)
 402  Format (1x," Nx,Ny       ",I3,1x,I3)
 403  Format (1x," Viscosity   ",f10.8)
 404  Format (1x," Density     ",f10.8)
 405  Format (1x," Dt          ",f10.8)
 409  Format (1x," Itermax     ",I10)
 410  Format (1x," Tolerance   ",f15.10)
 411  Format (1x," Nphi        ",I5)
 412  Format (1x," sliptol     ",f15.10)
 413  Format (1x," tend        ",f15.10)
 414  Format (1x," IPBC        ",I5)
 415  Format (1x," IGhia       ",I5)
 420  Format (1x," Impl        ",I5)
 421  Format (1x," Delh        ",f10.5)

      Stop
      End
